/*
 * FileManager.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.File;
import java.net.*;

/**
 * FileManager uses the SortedTreeNode class to keep que current directory (with
 * filtred files) in order.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class FileManager {
    
    static int FILE_TYPE = 0;
    static int SMBFILE_TYPE = 1;
    
    /* to know if FileManager is working with files or smbfiles */
    private int urlType = -1;    
    
    /* the sorted files tree */
    private SortedTreeNode filesNodeTree;
    
    /* URL array of files */
    private URL[] treeFiles;
    

    /**
     * Constructs the file manager from a URL array.
     * @param files The array of URLs to create the tree with.
     */
    public FileManager(URL[] files) {
        treeFiles = files;
        if (treeFiles != null) {
            UrlUtils urlUtils = new UrlUtils();
            urlType = urlUtils.getLocation(treeFiles[0]);
            /*
            jcifs.Config.registerSmbURLHandler();
            String protocol = treeFiles[0].getProtocol();
            if (protocol.equals("file")) {
                urlType = FILE_TYPE;
            } else if (protocol.equals("smb")) {
                urlType = SMBFILE_TYPE;
            }
             */
        }        
        createTreeNode();
    }

    /** Gets the url type, it can be File or SmbFile 
     */
    public int getUrlType() {
        return urlType;
    }
    
    /**
     * Changes to a new URL array and the tree node.
     * @param files The new array of URLs to create the tree with.
     */        
    public void changeTreeNode(URL[] files) {
        treeFiles = files;
        createTreeNode();        
    }

    /**
     * Generates the sorted tree node.
     */ 
    private void createTreeNode() {
        filesNodeTree = new SortedTreeNode(urlType);
	SortedTreeNode newNode;
	for ( int i = 0; i < treeFiles.length; i++ ) {
            newNode = new SortedTreeNode(treeFiles[i]); //posar fileType XXX
      	    filesNodeTree.add(newNode);
    	}        
    }

    /**
     * Returns true if at the index position there is a directory.
     * @param index     The index position of a node.
     * @return      true if at the index position there is a directory.
     */     
    public boolean isDirectoryNode(int index) {
        SortedTreeNode tmp = (SortedTreeNode) filesNodeTree.getChildAt( index );
        return tmp.getAllowsChildren();  
    }
    
    /**
     * Gets the number of nodes of the sorted tree.
     * @return      The number of nodes.
     */        
    public int getNodesNumber() {
        int numNodes;
        numNodes = filesNodeTree.getChildCount(); 
        return numNodes;
    }

    /**
     * Gets the file path of a node at index position.
     * @param index The index position of a node.
     * @return      The file path.
     */            
    public String getFileNameAt( int index ) {
        SortedTreeNode tmp = (SortedTreeNode) filesNodeTree.getChildAt( index );
        String fileName = getFullPathname(tmp);
        //System.out.println("ThumbnailManager - getFileAt() - (SortedTreeNode)getChildAt(): " + fileName);                                 
        return fileName;
    }
    
    /**
     * Gets the URL at the certain position of the tree.
     * @param index The position of the URL in the tree.
     * @return      The file.
     */            
    public URL getFileAt( int index ) {
        URL file = null;
        String fileName = getFileNameAt(index);
        try {        
            file = new URL(fileName);             
        } catch(MalformedURLException e) {
            System.out.println("ThumbnailFacade - getFileAt()");            
            System.out.println("MalformedURLException " + e);
       } 
       return file;
    }     
    
    /**
     * Gets the full path name of a node.
     * @param node  A node of a sortedtree.
     * @return      The full path name.
     */        
    private static String getFullPathname( SortedTreeNode node ) {
        if (node == null) return null;

        Object[] userObjectPath = node.getUserObjectPath();
        StringBuffer sb = new StringBuffer( "" );
        
        /* don't include "root" node at index 0 */
        for( int i=1; i < userObjectPath.length; i++ ) {
          sb.append( userObjectPath[i] );
          if (i < userObjectPath.length-1) {
                    sb.append( "/" );
            }
        }        
        return sb.toString();
   }    
}