/*
 * SuportedFilesFileFilter.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.files;

import java.io.*;
import javax.swing.*;
import java.net.*;

import jcifs.smb.*;

/**
 * SuportedFilesFileFilter is an instance of the FileFilter interface
 * Aceept all directories and image, audio and video formats that our plugins can display.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 20 Jun 2007
 */
public class SuportedFilesFileFilter implements FileFilter, SmbFileFilter {
    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;        
    
   private ImageFileFilter imageFileFilter = new ImageFileFilter();
   private AudioFileFilter audioFileFilter = new AudioFileFilter();
   private VideoFileFilter videoFileFilter = new VideoFileFilter();

    /**
     * Accept all directories and all fileformats accepted by the image, audio and video filefilters.     
     * @param url  The URL to accept.
     * @return      True if its accepted.
     */               
    public boolean accept(URL url) {
        boolean isAccepted = false; 
        UrlUtils urlUtils = new UrlUtils();
        int filesystemLocation = urlUtils.getLocation(url);
        if (filesystemLocation == LOCAL_FILE) {
            File file = urlUtils.getFile(url);
            isAccepted = accept(file);
        } else if (filesystemLocation == REMOTE_SMBFILE) {
            SmbFile smbfile = urlUtils.getSmbFile(url);
            isAccepted = accept(smbfile);          
        }
        return isAccepted;
    }
    
    /**
     * Accept all directories and all fileformats accepted by the image, audio and video filefilters.
     * @param file  The file to accept.
     * @return      True if its accepted.
     */       
    public boolean accept(File file) {
         if (imageFileFilter.accept(file) || audioFileFilter.accept(file) || videoFileFilter.accept(file)) {
             return true;
         } else {
             return false;
         }
    }

    /**
     * Accept all directories and all fileformats accepted by the image, audio and video filefilters.
     * @param file  The SmbFile to accept.
     * @return      True if its accepted.
     */         
    public boolean accept(SmbFile file) {
         if (imageFileFilter.accept(file) || audioFileFilter.accept(file) || videoFileFilter.accept(file)) {
             return true;
         } else {
             return false;
         }
    }    
}