/*
 * DVDCDJPanel.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.plugingui;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import java.net.*;

import jcifs.smb.*;

import lhmc.files.*;
import lhmc.plugins.*;
import lhmc.utils.*;
import lhmc.gui.util.*;
import lhmc.gui.plugingui.*;
import lhmc.gui.thumbnailsbrowser.*;

/**
 * DVD CD reproduction panel.
 * Depending on the type of DVD or CD it reproduces automaticaly or
 * displays a panel with an option list buttons and a browser panel 
 * with thumbnails.
 *
 * @version 	0.01 20 Jun 2007
 * @author 	Cristina Roura Claver
 */
public class DVDCDJPanel extends PluginGUI implements ActionListener {

    /* thumbnails browse panel */
    protected  ThumbnailsBrowser filesPanel;
    private JPanel opticalDiscPanel;
       
    /**
     * Constructs the DVDCD menu.
     */
    public DVDCDJPanel() {	
	appTitle = "DVD / CD";
        
	this.setLayout(new BorderLayout());
    }
    
    /**
     * Creates the movies panel.
     */         
    public void createPanel() {
        System.out.println("CDDVD JPANEL: "); 
    }   
    
    /**
     * Creates all the components of the DVD CD browser panel if its necessari.
     * Is composed by a buttons list and an thumbnail browser.
     * @return      The panel.
     */  
    private JPanel createComponents() {
        System.out.println("CDDVD JPANEL: CreateComponents");
	JPanel pane = new JPanel( );
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
        
        OpticalDiscTypeFinder opticalDiscTypeFinder = new OpticalDiscTypeFinder();
        int discType = opticalDiscTypeFinder.getDiscType();                   
            
        if (discType == opticalDiscTypeFinder.getTypeOf("DATA_CD") || discType == opticalDiscTypeFinder.getTypeOf("DATA_DVD")) {

            /* creates the buttons panel */
            String[] optionsName = { "Slideshow Images", "Play all music" };
            JPanel optionsPanel = createOptionsPanel(optionsName);
            optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
            pane.add( optionsPanel);
                
            /* creates the browser panel */
            try {
                URL root = new URL("file://" + getMediaUrl());            
            
                SuportedFilesFileFilter suportedFilesFileFilter = new SuportedFilesFileFilter();
                filesPanel = new ThumbnailsBrowser(root, suportedFilesFileFilter, getPluginFactory(), getMainFrame());        
                filesPanel.setAlignmentY(Component.CENTER_ALIGNMENT);
                pane.add( filesPanel);	
            } catch(MalformedURLException e) {
                System.out.println("ImagesJPanel - createComponents()");
                System.out.println("MalformedURLException " + e);
            }                
            return pane;                   
        } else {
            JLabel discLabel = new JLabel("");
            if (discType == opticalDiscTypeFinder.getTypeOf("NO_DISC")) {
                /* creates the buttons panel */
                String[] optionsName = { };
                JPanel optionsPanel = createOptionsPanel(optionsName);
                optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
                pane.add( optionsPanel);
                
                discLabel.setText("NO DISC");                 
                pane.add(discLabel);
            } else if (discType == opticalDiscTypeFinder.getTypeOf("AUDIO_CD")) {

                /* creates the buttons panel */
                String[] optionsName = { "Play Audio CD" };
                JPanel optionsPanel = createOptionsPanel(optionsName);
                optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
                pane.add( optionsPanel);
                
                discLabel.setText("EXECUTE AUDIO CD");
                Plugin audioCdPlugin = getPluginFactory().getPlugin(opticalDiscTypeFinder.getTypeOf("AUDIO_CD"));
                audioCdPlugin.setMainFrame(getMainFrame());
                audioCdPlugin.play(null);                                       
            } else if (discType == opticalDiscTypeFinder.getTypeOf("VIDEO_DVD")) {
                /* creates the buttons panel */
                String[] optionsName = { "Play DVD" };
                JPanel optionsPanel = createOptionsPanel(optionsName);
                optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
                pane.add( optionsPanel);
                
                discLabel.setText("EXECUTE VIDEO DVD");
                Plugin audioCdPlugin = getPluginFactory().getPlugin(opticalDiscTypeFinder.getTypeOf("VIDEO_DVD"));
                audioCdPlugin.setMainFrame(getMainFrame());
                audioCdPlugin.play(null);                  
                pane.add(discLabel);           
            }                
            return pane;
        }
        
    }

    /**
     * Creates all the option buttons.
     * @param   optionsName the buttons tu create.
     * @return      The buttons panel.
     */            
    private JPanel createOptionsPanel(String[] optionsName) {                
        
        JPanel pane = new JPanel(new GridLayout(optionsName.length + 1, 0));

        /* buttons creation */
        for (int i = 0 ; i < optionsName.length; i++) {
            /* creates a new button */
            CustomButton button = new CustomButton();
            button.setText(optionsName[i]);
            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {       /* is the first button */
                button.requestFocusInWindow();                 
            }            
            pane.add(button);   /* adds button to the panel */
	}
        focusedButton = backButton;
        pane.add(backButton);
        return pane;
    }
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        if (text == "Slideshow Images") {
            Plugin imagesPlugin = getPluginFactory().getPlugin(filesPanel.getActualRoot());
            imagesPlugin.setMainFrame(getMainFrame());
            imagesPlugin.playAll(filesPanel.getActualRoot());                           
        } else if (text == "Play all music") {
            Plugin musicPlugin = getPluginFactory().getPlugin(filesPanel.getActualRoot());
            musicPlugin.setMainFrame(getMainFrame());
            musicPlugin.playAll(filesPanel.getActualRoot());                        
        }
    }
    
    /**
     * Requests the focus for the initial focused component.
     */        
    public void focusComponent() {
        removeAll();
        opticalDiscPanel = createComponents();
        add(opticalDiscPanel, BorderLayout.CENTER);  
        super.focusComponent();                
    }         
}
