/*
 * ImagesJPanel.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package lhmc.gui.plugingui;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.text.DateFormat;
import java.util.Date;
import java.util.Calendar;
import java.util.Arrays;

import jcifs.smb.*;

import lhmc.gui.util.*;
import lhmc.gui.thumbnailsbrowser.*;
import lhmc.gui.plugingui.*;
import lhmc.files.*;
import lhmc.plugins.*;
import lhmc.gui.LHMCGUI;
import lhmc.utils.*;

/**
 * Images viewer panel.
 * Has a option list buttons and a browser panel with thumbnails.
 *
 * @version 	0.01 Jun 2007
 * @author 	Cristina Roura Claver
 */
public class ImagesJPanel extends PluginGUI implements ActionListener {
        
    
    /* thumbnails browse panel */
    protected ThumbnailsBrowser filesPanel;  
    
    String filesystemLocation = null;
    
    ImageFileFilter imageFileFilter = null;
    
    
    /**
     * Constructs the images viewer panel.
     */
    public ImagesJPanel() {    
        
        appTitle = "View Images";
        
        filesystemLocation = "Home";       
                
	setLayout(new BorderLayout());                                	
    }
    
    
    /**
     * Creates the images viewer panel.
     */       
    public void createPanel() {
        System.out.println("IMAGES JPANEL: Creating panel");
        add(createComponents(), BorderLayout.CENTER);  
    }    
    
    /**
     * Creates all the components of the image viewer.
     * Is composed by a buttons list and an thumbnail browser.
     * @return      The image viewer panel.
     */    
    private JPanel createComponents() {
	JPanel pane = new JPanel( );
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
	
        /* creates the buttons panel */
        JPanel optionsPanel = createOptionsPanel();
	optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
	pane.add( optionsPanel);

        try {
            URL root = new URL("file://" + getMediaUrl());        
            /* creates the browser panel */
            imageFileFilter = new ImageFileFilter();
            filesPanel = new ThumbnailsBrowser(root, imageFileFilter, getPluginFactory(), getMainFrame());
            filesPanel.setAlignmentY(Component.CENTER_ALIGNMENT);
            pane.add( filesPanel);	          
        } catch(MalformedURLException e) {
            System.out.println("ImagesJPanel - createComponents()");
            System.out.println("MalformedURLException " + e);
       }
        return pane;
    }

    /**
     * Creates all the option buttons.
     * @return      The buttons panel.
     */        
    private JPanel createOptionsPanel() {
        String[] optionsName = {  "Slideshow", "Change location" , "Save to..." };
        
        JPanel pane = new JPanel(new GridLayout(optionsName.length + 1, 0));
        
         /* buttons creation */
        for (int i = 0 ; i < optionsName.length; i++) {
            /* creates a new button */
            CustomButton button = new CustomButton();
            button.setText(optionsName[i]);
            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {       /* is the first button */
                button.requestFocusInWindow(); 
                focusedButton = button;
            }
            pane.add(button);   /* adds button to the panel */
	}
        pane.add(backButton);
        return pane;
    }                    

    /**
     * Requests the focus for the current thumbnail.
     */        
    public void focusThumbnail() {
        filesPanel.getFocusedThumbnailPanel().requestFocusInWindow();
    }
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        if (text == "Slideshow") {
            Plugin imagesPlugin = getPluginFactory().getPlugin(filesPanel.getActualRoot());
            imagesPlugin.setMainFrame(getMainFrame());
            imagesPlugin.playAll(filesPanel.getActualRoot());                                                                                                                        
        } else if (text == "Change location") {                 
            SharedFileSystemsDialog dialog= new SharedFileSystemsDialog(getMainFrame());
            dialog.createHostsDialog(filesystemLocation);
            dialog.pack();
            dialog.setLocationRelativeTo(null);
            dialog.setVisible(true);
                      
            String oldFilesystemLocation = filesystemLocation;
            filesystemLocation = dialog.getRemoteHost();
            if (filesystemLocation == "Home") {
                /* changing to home */                
                try {
                    URL root = new URL("file://" + getMediaUrl());
                    /* set file panel to host files */
                    filesPanel.setInitialRoot(root);
                    filesPanel.createFilesPanel(root, LOCAL_FILE );                  
                } catch(MalformedURLException ex) {
                    System.out.println("ImagesJPanel - actionPerformed() - Dialog HOME");                    
                    System.out.println("MalformedURLException " + ex);
               }
            } else if (filesystemLocation == "Close") {
                filesystemLocation = oldFilesystemLocation;
            } else { 
                /* changing to Remote */
                try {
                    System.out.println("ImagesJPanel - actionPerformed() - Dialog CANVI A REMOTE: " + filesystemLocation);
                    URL root = new URL("smb:///"+filesystemLocation+"/images/");
                    /* set file panel to host files */
                    filesPanel.setInitialRoot(root);
                    filesPanel.createFilesPanel(root, REMOTE_SMBFILE);                
                } catch (MalformedURLException ex) {
                    System.out.println("ImagesJPanel - actionPerformed() - Dialog REMOTE");
                    System.err.println("Error: " + ex.getMessage());
                } catch (Exception ex) {
                   System.err.println("Error: " + ex.getMessage());
                }
            }                   
        } else if (text == "Save to...") {  
            Vector<URL> filesToSave = filesPanel.getSelectedThumbnails(); 
            
            SharedFileSystemsDialog dialog= new SharedFileSystemsDialog(getMainFrame());
            dialog.createToSaveDialog(filesystemLocation);
            dialog.pack();
            dialog.setLocationRelativeTo(null);
            dialog.setVisible(true);
            
            /* where to copy */
            String hostToSave = dialog.getRemoteHost();
            String hostType = dialog.getHostType();
                          
            if (hostType != "Close") {
                
                Keyboard keyboard= new Keyboard(getMainFrame());
                keyboard.pack();
                keyboard.setLocationRelativeTo(null);
                keyboard.setVisible(true);
            
                String newDirectory = keyboard.getNewDirectory();
                
                CopyManager copyManager = new CopyManager(filesystemLocation, hostToSave, hostType, getMediaUrl(), "/images/", imageFileFilter);
                copyManager.copyFiles(filesToSave, newDirectory);
            }           
        }
    }      
            
    /**
     * Requests the focus for the initial focused component.
     */        
    public void focusComponent() {
        try {
            URL root = new URL("file://" + getMediaUrl());
            filesPanel.createFilesPanel(root, LOCAL_FILE); 
            filesystemLocation = "Home";
            super.focusComponent();        
        } catch(MalformedURLException e) {
            System.out.println("MalformedURLException " + e);
       }
    }    
}