/*
 * PluginGUI.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.plugingui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import lhmc.gui.*;
import lhmc.gui.util.*;
import lhmc.plugins.*;
import lhmc.gui.LHMCGUI;
import lhmc.utils.*;

/**
 * Plugin GUI.
 * Each plugin inherits form this class, that contains the basics for 
 * the plugin panel.
 *
 * @version 	0.01 Jun 2007
 * @author 	Cristina Roura Claver
 */
public class PluginGUI extends JPanel implements FocusListener {
    
    /* Colors */
    private  Color foregroundColor;
    private  Color highlightColor;
    
    protected static final int USB_DEVICES = 1;
    protected static final int CD_DVD_DEVICE = 2;   
    
    protected static final int LOCAL_FILE = 0;
    protected static final int REMOTE_SMBFILE = 1;    
       
    /* main frame pointer */
    private LHMCGUI mainFrame = null;

    /* initial focus component */
    protected CustomButton focusedButton;
    
    /* Title of the plugin */
    protected String appTitle;
    
    /* The back button */
    protected CustomButton backButton;
    
    /* Necesary to execute the apropiate plugin */
    private PluginFactory pluginFactory;
    
    /* The location of the files to browse with a plugin */
    private String mediaURL;
    

    /**
     * Constructs a basic plugin panel.
     */
    public PluginGUI() {

                
        loadColorTheme();
        
        pluginFactory = new PluginFactory();
        
        /* All plugin panel must have a back button that returns to the main menu. */
	backButton = new CustomButton("Back");
        backButton.addFocusListener(this);
	backButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {    /* when action ocurs */
                /* shows the main menu */
		getMainFrame().backToMenu();
	    }
	});
    }
 
    /**
     * Creates the plugin panel.
     */   
   public void createPanel() {   
   }    
   
    /**
     * Gets the title of the plugin.
     * @return      The title of the plugin.
     */         
    public String getTitle() {
	return appTitle;
    }
    
    /**
     * Sets the title of the plugin.
     * @param title The title of the plugin.
     */         
    public void setTitle(String title) {
	appTitle = title;
    }
    
    /**
     * Gets the files location.
     * @return      The files location.
     */         
    public String getMediaUrl() {
	return mediaURL;
    }
    
    /**
     * Sets the files location.
     * @param url The files location.
     */         
    public void setMediaUrl(String url) {
	mediaURL = url;
    }    
    
    /**
     * Sets the main frame to mainFrame.
     * @param mainFrame The main frame of the LHMCGUI.
     */    
    public void setMainFrame(LHMCGUI mainFrame) {
	this.mainFrame = mainFrame;
    }

    /**
     * Gets the main frame to mainFrame.
     * @return      The main frame of the LHMCGUI.
     * @see         LHMCGUI
     */    
    public LHMCGUI getMainFrame() {
	return mainFrame;
    }

    /**
     * Gets the plugin factory.
     * @return      The plugin factory.
     * @see         PluginFactory
     */    
    public PluginFactory getPluginFactory() {
	return pluginFactory;
    }
     
    /**
     * S the color theme necessari from the config class.
     */      
    private void loadColorTheme() {
        ConfigReader config = new ConfigReader();        
        highlightColor = config.loadColor("highlight");        
        foregroundColor = config.loadColor("foreground");          
    }
    
    /**
     * Requests the focus for the initial focused component.
     */        
    public void focusComponent() {
        focusedButton.requestFocusInWindow();
    } 
           
    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */           
     public void focusGained(FocusEvent e) {
        e.getComponent().setForeground(highlightColor);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */                
    public void focusLost(FocusEvent e) {
        e.getComponent().setForeground(foregroundColor);
    }
}