/*
 * RemovableDevJPanel.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.plugingui;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.net.*;
import java.util.Vector;

import lhmc.files.*;
import lhmc.plugins.*;
import lhmc.utils.*;
import lhmc.gui.util.*;
import lhmc.gui.plugingui.*;
import lhmc.gui.thumbnailsbrowser.*;

/**
 * Removable Devices panel.
 * Has a option list buttons and a browser panel with thumbnails.
 *
 * @version 	0.01 Jun 2007
 * @author 	Cristina Roura Claver
 */
public class RemovableDevJPanel extends PluginGUI implements ActionListener {

    /* thumbnails browse panel */
    protected  ThumbnailsBrowser filesPanel;  
    
    private JPanel mainPanel;
    private URL root = null;
    private URL[] usbDevices = null;
    
    String filesystemLocation = null;
    
    SuportedFilesFileFilter suportedFilesFileFilter = null;    
    
    private RemovableDevicesActivator removableDevicesActivator = null;
   
    /**
     * Constructs the removable devices panel
     */
    public RemovableDevJPanel() {
	appTitle = "Removable Devices";
	setLayout(new BorderLayout());               
    }
    
   
    /**
     * Creates the movies panel.
     */         
    public void createPanel() {
        System.out.println("RemovableDevJPanel - Creating panel");
        try {
            root =  new URL("file://" + getMediaUrl());
        } catch(MalformedURLException e) {
            System.out.println("MalformedURLException " + e);
        }
        filesystemLocation = root.getPath();
        removableDevicesActivator = new RemovableDevicesActivator(root); 
        usbDevices = removableDevicesActivator.getRemovableDevices();
        add(createComponents(), BorderLayout.CENTER);  
    }         
    
    /**
     * Creates all the components of the removable devices panel.
     * Is composed by a buttons list and an thumbnail browser.
     * @return      The removibles panel.
     */      
    public JPanel createComponents() {

        /*
        if (usbDevices.length == 0) {
            System.out.println("NO USB DEVICES");
            usbDevices = new URL[1];
            usbDevices[0] = root;
        }
        */        
	JPanel pane = new JPanel( );
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
	
        /* creates the buttons panel */
        JPanel optionsPanel = createOptionsPanel();
	optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
	pane.add( optionsPanel);
               
        /* creates the browser panel */
        suportedFilesFileFilter = new SuportedFilesFileFilter();

        filesPanel = new ThumbnailsBrowser(root, usbDevices, suportedFilesFileFilter, getPluginFactory(), getMainFrame());
	filesPanel.setAlignmentY(Component.CENTER_ALIGNMENT);                
	pane.add( filesPanel);	
	return pane;  
    }

    /**
     * Creates all the option buttons.
     * @return      The buttons panel.
     */                
    public JPanel createOptionsPanel() {
        String[] optionsName = {  "Slideshow Images", "Play all music", "Save to..." };
        
        JPanel pane = new JPanel(new GridLayout(optionsName.length + 1, 0));

        /* buttons creation */
        for (int i = 0 ; i < optionsName.length; i++) {
            /* creates a new button */
            CustomButton button = new CustomButton();
            button.setText(optionsName[i]);
            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {       /* is the first button */
                button.requestFocusInWindow(); 
                focusedButton = button;
            }
            pane.add(button);   /* adds button to the panel */
	}
        pane.add(backButton);
        return pane;
    }

    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        UrlUtils urlUtils = new UrlUtils(filesPanel.getActualRoot());
        if (text == "Slideshow Images") {
            ImageFileFilter imageFileFilter = new ImageFileFilter();
            URL[] imagesFiles = urlUtils.getFilteredUrls(filesPanel.getActualRoot(), imageFileFilter);
            if (imagesFiles != null) {
                Plugin plugin = getPluginFactory().getPlugin(imagesFiles[0]);
                plugin.setMainFrame(getMainFrame());
                plugin.playAll(filesPanel.getActualRoot());                           
            }
        } else if (text == "Play all music") {
            AudioFileFilter audioFileFilter = new AudioFileFilter();
            URL[] audioFiles = urlUtils.getFilteredUrls(filesPanel.getActualRoot(), audioFileFilter);            
            if (audioFiles != null) {
                Plugin plugin = getPluginFactory().getPlugin(audioFiles[0]);
                plugin.setMainFrame(getMainFrame());
                plugin.playAll(filesPanel.getActualRoot());                                        
            }
        } else if (text == "Save to...") {  
            filesystemLocation = filesPanel.getActualRoot().getPath();
            Vector<URL> filesToSave = filesPanel.getSelectedThumbnails();
            
            SharedFileSystemsDialog dialog= new SharedFileSystemsDialog(getMainFrame());
            dialog.createToSaveDialog(filesystemLocation);
            dialog.pack();
            dialog.setLocationRelativeTo(null);
            dialog.setVisible(true);
            
            String hostToSave = dialog.getRemoteHost();
            String hostType = dialog.getHostType();
                          
            if (hostType != "Close") {
                
                Keyboard keyboard= new Keyboard(getMainFrame());
                keyboard.pack();
                keyboard.setLocationRelativeTo(null);
                keyboard.setVisible(true);
            
                String newDirectory = keyboard.getNewDirectory();
                
                System.out.println("filesystemLocation: " + filesystemLocation + " hostToSave: " + hostToSave +" hostType: "+ hostType);
                
                CopyManager copyManager = new CopyManager(filesystemLocation, hostToSave, hostType, getMediaUrl(), "", suportedFilesFileFilter);
                copyManager.copyFiles(filesToSave, newDirectory);
            }           
        }
    }    

    /**
     * Requests the focus for the initial focused component.
     */        
    public void focusComponent() {
        removableDevicesActivator.activatingUSBDevices();
        usbDevices = removableDevicesActivator.getRemovableDevices();
        filesPanel.setUsbDevices(usbDevices);
        filesPanel.createFilesPanel(root, LOCAL_FILE);
        backButton.requestFocusInWindow(); 
        super.focusComponent();        
    }
    

    
}
