/*
 * TvJPanel.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.plugingui;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;

import lhmc.gui.util.*;
import lhmc.gui.plugingui.*;
import lhmc.plugins.*;

/**
 * Television viewer panel.
 * Has a option list buttons.
 *
 * @version 	0.01 Jun 2007
 * @author 	Cristina Roura Claver
 */
public class TvJPanel extends PluginGUI implements ActionListener {
     private int WATCH_TV   = 6;
     private int RECORD_TV = 7;
     
     private JPanel optionsPanel;
     
   /**
     * Constructs the television viewer panel.
     */
    public TvJPanel() {	
	appTitle = "Watch TV";
        
	setLayout(new BorderLayout());        
    }
    
    
    /**
     * Creates the television viewer panel.
     */         
    public void createPanel() {
        System.out.println("TV JPANEL: Creating panel");
        add(createComponents(), BorderLayout.CENTER);  
    }   
    
    /**
     * Creates all the components of the television viewer panel.
     * Is composed by a buttons list.
     * @return      The television viewer panel.
     */            
    public JPanel createComponents() {
	JPanel pane = new JPanel( );
	pane.setLayout(new BoxLayout(pane, BoxLayout.X_AXIS));
	
        /* creates the buttons panel */
        optionsPanel = createOptionsPanel();
	optionsPanel.setAlignmentY(Component.LEFT_ALIGNMENT);
	pane.add( optionsPanel);

	return pane;
    }

    /**
     * Creates all the option buttons.
     * @return      The buttons panel.
     */                
    public JPanel createOptionsPanel() {
        String[] optionsName = {  "Watch TDT" , "Record TDT", "Stop Recording TDT" };        
         
        JPanel pane = new JPanel(new GridLayout(optionsName.length + 1, 0));
        
         /* buttons creation */
        for (int i = 0 ; i < optionsName.length; i++) {
            /* creates a new button */
            CustomButton button = new CustomButton();
            button.setText(optionsName[i]);
            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {       /* is the first button */
                button.requestFocusInWindow(); 
                focusedButton = button;
            }
            pane.add(button);   /* adds button to the panel */
	}
        pane.add(backButton);
        return pane;
    }    

    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        if ( text == "Watch TDT" ) {   
 	    if (getMainFrame().getIsRecordingDvbt() == true) {
		NoticeDialog notice = new NoticeDialog(getMainFrame(), "Option not available: The system is actually recording.");
                notice.pack();
                notice.setLocationRelativeTo(null);
                notice.setVisible(true);  
            } else {
            	Plugin tvPlugin = getPluginFactory().getPlugin(WATCH_TV);
            	tvPlugin.setMainFrame(getMainFrame());
            	tvPlugin.play(null);                                
            }
         

        } else if ( text == "Record TDT" ) {      
            if (getMainFrame().getIsRecordingDvbt() == false) {
                Plugin tvPlugin = getPluginFactory().getPlugin(RECORD_TV);
                tvPlugin.setMainFrame(getMainFrame());
                tvPlugin.play(null);
            } else {
                NoticeDialog notice = new NoticeDialog(getMainFrame(), "Option not available: The system is actually recording.");
                notice.pack();
                notice.setLocationRelativeTo(null);
                notice.setVisible(true);                
            }
        } else if (text == "Stop Recording TDT") {
            if (getMainFrame().getIsRecordingDvbt() == true) {
                getMainFrame().getRecordingThread().stop();
                getMainFrame().setIsRecordingDvbt(false, null);
                NoticeDialog notice = new NoticeDialog(getMainFrame(), "Record stoped. Show stored in movies directory.");
                notice.pack();
                notice.setLocationRelativeTo(null);
                notice.setVisible(true);   
            } else {
                NoticeDialog notice = new NoticeDialog(getMainFrame(), "Option not available: The system is not recording.");
                notice.pack();
                notice.setLocationRelativeTo(null);
                notice.setVisible(true);                  
            }
            
        } 
    }       
}
