/*
 * ThumbnailPanel.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.thumbnailsbrowser;
        
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import java.net.*;

import lhmc.utils.*;

/**
 * ThumbnaiPanel is the external panel that contains the thumbnail and
 * adds a border.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class ThumbnailPanel extends JPanel {

    /* Color */
    private Color backgroundColor;
    
    private URL thumbnailPath;
    
    private int thumbPanelSize = 220;
    
    /**
     * Constructs the thumbanail panel
     */    
    public ThumbnailPanel() {
        super();
        
        loadColorTheme();
        
        setLayout(new BorderLayout());
        setPreferredSize(new Dimension(thumbPanelSize,thumbPanelSize));
        setBorder(new EmptyBorder(10,10,0,10));
        setBackground(backgroundColor);
    }

    /**
     * Sets the path of the thumbnail.
     * @param filePath The file of the thumbnail.
     */       
    public void setThumbnailPath(URL filePath) {
        thumbnailPath = filePath;
    }

    /**
     * Gets the path of the thumbnail.
     * @return      The url file of the thumbnail.
     */    
    public URL getThumbnailPath() {
        return thumbnailPath;
    }
    
    /**
     * Loads the colors the config file.
     */      
    private void loadColorTheme() {
        ConfigReader config = new ConfigReader();        
        backgroundColor = config.loadColor("background"); 
    }        
}