/*
 * ThumbnailsBrowser.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.thumbnailsbrowser;

import java.io.File;
import java.awt.event.KeyEvent;
import javax.swing.*;  
import java.awt.*;
import java.awt.event.*;
import java.io.FileFilter;
import java.io.*;
import java.util.*;
import java.net.*;
import java.util.Vector;

import jcifs.smb.*;

import lhmc.plugins.*;
import lhmc.gui.util.*;
import lhmc.files.*;
import lhmc.gui.LHMCGUI;
import lhmc.plugins.*;
import lhmc.utils.*;

/**
 * ThumbnailsBrowser its a JScrollPane that contains thumbnails corresponding
 * to the files in a especified directory.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 20 Jun 2007
 */
public class ThumbnailsBrowser extends JScrollPane {
    
    /* Colors */
    private Color backgroundColor;
    private Color highlightColor;  
    private Color selectedColor;
    
    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;    
    
    
    /* The selected thumbnail */
    protected URL selectedThumbnail = null;
    
    /* mantains the directory treenode and the thumbnails displayed */
    protected ThumbnailFacade thumbnailFacade;
    
    /* to execute a file with the correspondig plugin */
    protected PluginFactory pluginFactory;
    
    /* the initial directory */
    private URL initialRoot = null;    
    
    /* the actual browsing directory */
    private URL actualRoot = null;
    
    private JPanel thumbnailsPanel;
    
    private FileFilter customFileFilter;
    
    protected LHMCGUI mainFrame;
    
    protected JPanel focusedThumbnailPanel;    
    
    private URL[] usbDevices = null;
    
    private ConfigReader config = new ConfigReader();
    
    private int actualFilesystemLocation = -1;

    private Vector<URL> selectedThumbnails = new Vector<URL>();

            
    /**
     * Constructs the thumbanail browser panel.
     * @param rootFile      the current directory to browse.
     * @param fileFilter    the file filter to apply if the all files are from the same kind.
     * @param pluginFactory the pluginFactory to instance plugins.
     * @param mainFrame     the mainFrame of the application.
     */        
    public ThumbnailsBrowser( URL rootFile, FileFilter fileFilter, PluginFactory pluginFactory, LHMCGUI mainFrame) { 
     
        setAttributes(rootFile, fileFilter, pluginFactory, mainFrame);        
        /* creates the panel that contains all thumbnails from a directory */
        createFilesPanel(rootFile, LOCAL_FILE);                       
    }
        
    /**
     * Constructs the thumbanail browser panel for removable devices.
     * @param rootFile              the current directory to browse.
     * @param initialRemovableFiles url file array of removable devices.
     * @param fileFilter            the file filter to apply if the all files are from the same kind.
     * @param pluginFactory         the pluginFactory to instance plugins.
     * @param mainFrame             the mainFrame of the application.
     */       
    public ThumbnailsBrowser( URL rootFile, URL[] initialRemovableFiles, FileFilter fileFilter, PluginFactory pluginFactory, LHMCGUI mainFrame ) { 

        setAttributes(rootFile, fileFilter, pluginFactory, mainFrame);
        usbDevices = initialRemovableFiles;   
        
        /* creates the panel that contains all thumbnails from a directory */
        createFilesPanel(rootFile, LOCAL_FILE);               
    } 
    
    /**
     * Sets the initial attributes to the class.
     * @param rootFile              the current directory to browse.
     * @param fileFilter            the file filter to apply if the all files are from the same kind.
     * @param pluginFactory         the pluginFactory to instance plugins.
     * @param mainFrame             the mainFrame of the application.
     */     
    private void setAttributes(URL rootFile, FileFilter fileFilter, PluginFactory pluginFactory, LHMCGUI mainFrame) {
        loadColorTheme();
        
        initialRoot = rootFile;
        this.customFileFilter = fileFilter;
       
        selectedThumbnail = rootFile;

        this.pluginFactory = pluginFactory;
        this.mainFrame = mainFrame;

        setOpaque(false);
        getViewport().setOpaque(false);
        setViewportBorder(BorderFactory.createEmptyBorder());
        setBorder(BorderFactory.createEmptyBorder());   
        
    }

    /**
     * Gets the URL file of the selected thumbnail.
     * @return      The url file correspondig to the thumbnail.
     */        
    public URL getSelectedThumbnail() {        
        return selectedThumbnail;
    }

    /**
     * Gets current directory.
     * @return      The URL file of the actual root.
     */      
    public URL getActualRoot() {
        return actualRoot;        
    }

    /**
     * Sets current directory.
     * @param root  The url file of the actual root.
     */      
    public void setActualRoot(URL root) {
        actualRoot = root;
    }

    /**
     * Sets the initial root directory.
     * @param root  The url file of the new root directory.
     */      
    public void setInitialRoot(URL root) {
        initialRoot = root;
    }       

    /**
     * Gets the initial root directory.
     * @return      The url file of the root directory.
     */      
    public URL getInitialRoot() {
        return initialRoot;        
    }
    
    /**
     * Sets the removible devices directories.
     * @param usbDevices  The url files of the removible devices.
     */      
    public void setUsbDevices(URL[] usbDevices) {
        this.usbDevices = usbDevices;
    }     

    /**
     * Gets a vector of URL files from the selected thumbnails.
     * @return      The url files vector correspondig to the selected thumbnails.
     */       
    public Vector<URL> getSelectedThumbnails() {
        return selectedThumbnails;
    }
    
    /**
     * Gets the actual filesystem location
     * @return      The actual filesystem location.
     */        
    public int getActualFilesystemLocation() {
        return actualFilesystemLocation;
    }
    
    /**
     * Creates a panel of thumbnails. Each thumbnail corresponds to a directory,
     * a file or a back image.
     * @param rootFile  The current directory.
     * @param urlType   the type of url. 
     */        
    public void createFilesPanel(URL rootFile, int urlType) { 
            
        /* individual thumbnail panel size */
        int thumbPanelSize = 220;

        /* number of thumbnails */
        int numThumbnails = 0;
        
        /* number of columns and rows for the GridLayout Panel */
        int numColumns =  3;
        int numRows;
        
        /* GridLayout Panel size and spacing */
        int panelWidth;
        int panelHeight;
        int spacing = 10;
        
        selectedThumbnails.removeAllElements();
        
        actualFilesystemLocation = urlType;  
        actualRoot = rootFile;
        
        //System.out.println("ThumbnailsBrowser - createFilesPanel() - rootFile: " + rootFile.getPath());        
        //System.out.println("ThumbnailsBrowser - createFilesPanel() - actualRoot: " + actualRoot.getPath());        

        final UrlUtils urlUtils = new UrlUtils(actualFilesystemLocation);
        URL[] urls = null;

        /* if its a usb device panel and it's the initial panel */
        if (usbDevices != null) {
                if ((urlUtils.getURLParent(usbDevices[0])).getPath().equals(actualRoot.getPath())) {
                    urls = usbDevices;  
                    numThumbnails = urls.length;
                } else {
                    urls = urlUtils.getFilteredUrls(actualRoot, customFileFilter);
                    numThumbnails = urls.length + 1;
                }
        } else {
            urls = urlUtils.getFilteredUrls(actualRoot, customFileFilter);
            /* number of thumbnails = number of files + 1 (the back image) */
            if (!(initialRoot.getPath().equals(actualRoot.getPath()))) {
                numThumbnails = urls.length + 1;    
            } else {
                numThumbnails = urls.length;
            }   
        }
                              
        if (numThumbnails != 0) {

            /* calculates number of Rows depending on the number of Thumbnails and the default number of columns*/
            numRows = (int) Math.ceil(1.0 * numThumbnails / numColumns);   

            /* calculates the size of the panel */
            panelWidth = numColumns * (thumbPanelSize + spacing);
            panelHeight = numRows * (thumbPanelSize + spacing);

            /* setting the thumbnail Panel */
            thumbnailsPanel = new JPanel(new GridLayout(numRows, numColumns, spacing, spacing));        
            thumbnailsPanel.setPreferredSize(new Dimension(panelWidth, panelHeight));

            thumbnailFacade = new ThumbnailFacade(urls);

                /* if a back image is needed, create it */
                if (!(initialRoot.getPath().equals(actualRoot.getPath()))) {

                    /* creation of back image */
                    ImageJComponent backImage = new ImageJComponent(config.getProperty("IconBack"));

                    /* creation of back panel and insertion back image*/                
                    ThumbnailPanel backPanel = new ThumbnailPanel();
                    backPanel.add(backImage, BorderLayout.CENTER);

                    /* get the thumbanail file path and set it in the thumbanail Panel */                
                    URL thumbnailPath = urlUtils.getURLParent(actualRoot);
                    backPanel.setThumbnailPath(thumbnailPath);

                    /* focus listener for the Panel, highlight the panel and save the path of the selected panel */                
                    backPanel.setFocusable(true);
                    backPanel.addFocusListener(new FocusListener() {
                        public void focusGained(FocusEvent e) {
                            e.getComponent().setBackground(highlightColor);                    
                            ThumbnailPanel tp = (ThumbnailPanel) e.getSource();  
                            //System.out.println("ThumbnailsBrowser - createFilesPanel() - BACK createURL(): " + tp.getThumbnailPath());
                            selectedThumbnail = tp.getThumbnailPath();
                            focusedThumbnailPanel = tp;
                        }
                        public void focusLost(FocusEvent e) {
                            e.getComponent().setBackground(backgroundColor);
                        }
                    });

                    /* add a Key listener */
                    backPanel.addKeyListener(new KeyListener() {
                        public void keyPressed(KeyEvent e) {
                            int key = e.getKeyCode();
                            if (key == KeyEvent.VK_ENTER) {                                
                                /* gets the array of files from the back directory */
                                URL[] urlsPath = urlUtils.getFilteredUrls(selectedThumbnail, customFileFilter);
                                /* creates the new panel */
                                thumbnailFacade.changeDirectory(urlsPath);
                                createFilesPanel(selectedThumbnail, actualFilesystemLocation);
                            }
                            /* make sure the key isn't processed for anything else */
                            e.consume();                        
                        }
                        public void keyReleased(KeyEvent e) { 
                            /* make sure the key isn't processed for anything else */
                            e.consume();
                        }
                        public void keyTyped(KeyEvent e) { 
                            /* make sure the key isn't processed for anything else */
                            e.consume();
                        }
                    });

                    thumbnailsPanel.add(backPanel);
                }        

            /* for each file, create a thumbnail and put it in the thumbnailsPanel */
            for (int i = 0; i < urls.length; i++) {

                /* get the thumbnail image for this file */
                JPanel thumbnail = thumbnailFacade.createThumbnail(i);

                /* create the thumbnail panel */
                ThumbnailPanel thumbnailPanel = new ThumbnailPanel();

                /* get the thumbanail file and set the path in the thumbanail Panel */
                URL thumbFile = thumbnailFacade.getFileAt(i);
                //System.out.println("ThumbnailsBrowser - createFilesPanel() - thumbnailFacade.getFileAt(i): " + thumbFile );                    

                thumbnailPanel.setThumbnailPath(thumbFile);

                /* creates a label of the file name */
                JLabel fileNameLabel = new JLabel(urlUtils.getUrlName(thumbFile));

                /* insert the thumbnail and the label to the thumbnail panel */
                thumbnailPanel.add(thumbnail, BorderLayout.CENTER);
                thumbnailPanel.add(fileNameLabel, BorderLayout.SOUTH); 

                /* focus listener for the Panel, highlight the panel and save the path of the selected panel */
                thumbnailPanel.setFocusable(true);
                thumbnailPanel.addFocusListener(new FocusListener() {
                    public void focusGained(FocusEvent e) {
                        e.getComponent().setBackground(highlightColor);                    
                        ThumbnailPanel tp = (ThumbnailPanel) e.getSource();      
                        //System.out.println("ThumbnailsBrowser - createFilesPanel() - createURL(): " + tp.getThumbnailPath() );                    
                        selectedThumbnail = tp.getThumbnailPath();                    
                        focusedThumbnailPanel = tp;
                    }               

                    public void focusLost(FocusEvent e) {
                        ThumbnailPanel tp = (ThumbnailPanel) e.getSource();
                        if (!selectedThumbnails.isEmpty() ) {
                            if (selectedThumbnails.contains(tp.getThumbnailPath())) {
                                e.getComponent().setBackground(selectedColor);
                            } else {
                                e.getComponent().setBackground(backgroundColor);                                               
                            }
                        } else {                    
                            e.getComponent().setBackground(backgroundColor);                   
                        }

                    }
                });

               /* add a Key listener */
               URL actualFile = thumbFile;



               if (urlUtils.isUrlDirectory(actualFile)) {
                        /* if it is a directory show the thumbnails inside it */
                        thumbnailPanel.addKeyListener(new KeyListener() {
                            public void keyPressed(KeyEvent e) {
                                int key = e.getKeyCode();
                                if (key == KeyEvent.VK_ENTER) {
                                    /* gets the array of files from the directory */
                                    URL[] filesPath = urlUtils.getFilteredUrls(selectedThumbnail, customFileFilter);

                                    /* creates the new panel */
                                    thumbnailFacade.changeDirectory(filesPath);
                                    createFilesPanel(selectedThumbnail, actualFilesystemLocation);                        
                                } else if (key == KeyEvent.VK_SHIFT) {
                                    if (selectedThumbnails.contains(selectedThumbnail)) {
                                        selectedThumbnails.remove(selectedThumbnail);
                                    } else {
                                        selectedThumbnails.add(selectedThumbnail);
                                    }
                                }
                                /* make sure the key isn't processed for anything else */
                                e.consume();                        
                            }
                            public void keyReleased(KeyEvent e) { 
                                /* make sure the key isn't processed for anything else */
                                e.consume();                        
                            }
                            public void keyTyped(KeyEvent e) { 
                                /* make sure the key isn't processed for anything else */
                                e.consume();                        
                            }
                        });
                    } else {
                        /* if it is a thumbnail shows it */
                        thumbnailPanel.addKeyListener(new KeyListener() {
                            public void keyPressed(KeyEvent e) {
                                int key = e.getKeyCode();
                                if (key == KeyEvent.VK_ENTER) {                                
                                    Plugin plugin = pluginFactory.getPlugin(selectedThumbnail);
                                    plugin.setMainFrame(mainFrame);
                                    plugin.play(selectedThumbnail);
                                    getFocusedThumbnailPanel().requestFocusInWindow();
                                }  else if (key == KeyEvent.VK_SHIFT) {
                                    if (selectedThumbnails.contains(selectedThumbnail)) {
                                        selectedThumbnails.remove(selectedThumbnail);
                                    } else {
                                        selectedThumbnails.add(selectedThumbnail);
                                    }
                                }
                                /* make sure the key isn't processed for anything else */
                                e.consume();                                                
                            }
                            public void keyReleased(KeyEvent e) { 
                                /* make sure the key isn't processed for anything else */
                                e.consume();                        
                            }
                            public void keyTyped(KeyEvent e) { 
                                /* make sure the key isn't processed for anything else */
                                e.consume();                        
                            }
                        });                
                    }          
                /** adds the thumbnailPanel to the thumnails Panel */
                thumbnailsPanel.add(thumbnailPanel);
            }        
            /* insert thumbnails panel in the scrollPane */
        } else {
            thumbnailsPanel = new JPanel();
        }
        getViewport().add(thumbnailsPanel, null);
    }
    
    

    
    /**
     * Returns the current focused Thumbnail.
     * @return          The focused thumbnail panel.
     */     
    public JPanel getFocusedThumbnailPanel() {
        return focusedThumbnailPanel;
    }
    
    /**
     * Loads the colors from the config file.
     */     
    private void loadColorTheme() {
        highlightColor = config.loadColor("highlight");        
        backgroundColor = config.loadColor("background");             
        selectedColor = config.loadColor("selected");  
    }
       
}