/*
 * CustomButton.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.util;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import lhmc.utils.*;

/**
 * Custom button with concrete characteristics.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class CustomButton extends JButton {
    
    /* Font Types and Colors */
    private Font buttonFont;
    private Color foregroundColor;
    
    /* image representing the option of the button */
    private String imageURL; 

    /**
     * Constructs the custom button.
     */    
    public CustomButton() {
	super();        
        loadColorFontTheme();        
	setButtonProperties();
    }

    /**
     * Constructs the custom button.
     * @param name  The text of the button.
     */    
    public CustomButton(String name) {
        super(name);
        
        loadColorFontTheme();
        
	setButtonProperties();
    }

    /**
     * Sets the button characteristics and sets the keyactions.
     */        
    private void setButtonProperties() {	
        setContentAreaFilled(false);    /* not changin color when clicking it */
	setForeground(foregroundColor);
	setOpaque(true);
	setHorizontalAlignment(SwingConstants.CENTER);
	setFont(buttonFont);
	setBorderPainted(false);
	setFocusable(true);
	setFocusPainted(false);
	    
	/** Enter key behaves like space key */
	/** Set the InputMap that is used when the component has focus. */
	InputMap im = getInputMap();
	KeyStroke enter_pressed = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, false);
	KeyStroke enter_released = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, true);
	
	KeyStroke space_pressed = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, false);
	KeyStroke space_released = KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0, true);

	Object spaceKeyPressed = im.get(space_pressed);
	Object spaceKeyReleased = im.get(space_released);
	im.put(enter_pressed, spaceKeyPressed);
	im.put(enter_released, spaceKeyReleased);
    }

    /**
     * Gets the location of the image related to the button.
     * @return      the image path.
     */    
    public String getImageURL() {
        return imageURL;
        
    }

    /**
     * Sets the location of the image related to the button.
     * @param imgUrl    the image path.
     */        
    public void setImageURL(String imgUrl) {
        imageURL = imgUrl;
    } 

    /**
     * Loads the colors and fonts from the config file.
     */         
    private void loadColorFontTheme() {
        ConfigReader config = new ConfigReader();
        foregroundColor = config.loadColor("foreground");        
        buttonFont = config.loadFont("button");        
    }
}
