/*
 * ImageJComponent.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.util;
        
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;

/**
 * Loads an Image from an external file and converts it a JComponent
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class ImageJComponent extends JComponent {
   
    private BufferedImage img;
    
    private int defaultSize = 200;

    /**
     * Constructs a null image JComponent.
     */    
    public ImageJComponent() {
	img = null;
    }

    /**
     * Constructs an image JComponent from a File
     * @param file  The image file.
     */    
    public ImageJComponent(File file) {
	try {
	    img = ImageIO.read(file);
	} catch (IOException e) {
            System.out.println(e);
	}
    }

    /**
     * Constructs an image JComponent from a String path.
     * @param str  The path where the image located.
     */
    public ImageJComponent(String str) {
	try {
	    img = ImageIO.read(new File(str));
	} catch (IOException e) {
            System.out.println(e);            
	}
    }

    /**
     * Sets an image JComponent from a String path.
     * @param str  The path where the image located.
     */      
    public void setImage(String str) {
	try {
	    img = ImageIO.read(new File(str));
	    repaint();
	} catch (IOException e) {
            System.out.println(e);            
	}
    }

    /**
     * Gets the preferred Size of the image.
     * @return      The size of the image.
     */        
    public Dimension getPreferredSize() {
        if (img == null) {
             return new Dimension(defaultSize,defaultSize);
        } else {
           return new Dimension(img.getWidth(null), img.getHeight(null));
       }
    }

    /**
     * Reimplementation of paintComponent to paint the img.
     * @param g the Graphics context.
     */       
    protected void paintComponent(Graphics g) {
	g.drawImage(img, 0, 0, null);        
    }
    
}