/*
 * Keyboard.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.util;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import lhmc.utils.*;

/**
 * Keyboard.
 * Dialog with keybload letters and an input label to write a name.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class Keyboard extends JDialog implements FocusListener, ActionListener {
    
    /* Font Types and Colors */
    private Color foregroundColor;    
    private Color highlightColor; 
    private Font noticeFont;        

     JLabel directoryLabel = null;
     String directoryString = "";
    

    public final static String[] keys = { "A", "B", "C", "D", "E", "F", 
                                          "G", "H", "I", "J", "K", "L", "M", 
                                          "N", "O", "P", "Q", "R", "S", "T", 
                                          "U", "V", "W", "X", "Y", "Z", "1", 
                                          "2", "3","4","5","6","7","8", "9",
                                          "0", "_", "SPACE" };
    
    /**
     * Constructs the Keyboard dialog.
     */    
    public Keyboard(JFrame frame) {
        super(frame, true);
        loadColorFontTheme();
        setUndecorated(true);


        JPanel pane = new JPanel();
        pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
        pane.setBorder(BorderFactory.createLineBorder(foregroundColor,10));
        
        JLabel messageLabel = new JLabel("Insert the name of destination directory");
        messageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        messageLabel.setFont(noticeFont); 
        pane.add(messageLabel);        
        
        /* calculates number of Rows depending on the number of Thumbnails and the default number of columns*/
        int numColumns = 6;
        int numRows = (int) Math.ceil(1.0 * keys.length / numColumns);   
                        
        
	JPanel keyPanel = new JPanel(new GridLayout(numRows, numColumns));


        for (int i = 0 ; i < keys.length; i++) {
            //* creates a new button
            CustomButton button = new CustomButton();
            button.setText(keys[i]);

            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {
                button.requestFocusInWindow(); 
            }
            keyPanel.add(button);   //* adds button to the panel               
        }
        pane.add(keyPanel);     
        
        JPanel inputPanel = new JPanel(new FlowLayout());
    
        JLabel inputLabel = new JLabel("New directory: ");
        inputLabel.setFont(noticeFont); 
        inputPanel.add(inputLabel);         

        directoryLabel = new JLabel("");
        directoryLabel.setFont(noticeFont); 
        inputPanel.add(directoryLabel);  
        
        pane.add(inputPanel);

        
        /* creates the back button */
        CustomButton button = new CustomButton("OK");
	button.addFocusListener(this);
	button.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {    /* when action ocurs */
                /* shows the main menu */
		dispose();
	    }
	});
        pane.add(button);        
        
        setContentPane(pane);
    }

    /**
     * Gets the name wrote.
     * @return  the wrote name.
     */      
    public String getNewDirectory() {
        return directoryString;
    }
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText(); 
        if (text == "OK") {
            hide();
        } else {
            directoryString = directoryString + text;
            directoryLabel.setText(directoryString);
        }
    }
    
    /**
     * Loads the colors and fonts from the config file.
     */     
    private void loadColorFontTheme() {
        ConfigReader config = new ConfigReader();
        foregroundColor = config.loadColor("foreground");        
        highlightColor = config.loadColor("highlight"); 
        noticeFont = config.loadFont("button");
    }          

    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* highlights the current button */
        e.getComponent().setForeground(highlightColor);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* restore the default color */
        e.getComponent().setForeground(foregroundColor);
    } 
}
