/*
 * NoticeDialog.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.util;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

import lhmc.utils.*;

/**
 * Custom Dialog.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 20 Jun 2007
 */
public class NoticeDialog extends JDialog implements FocusListener {
    
    /* Font Types and Colors */
    private Color foregroundColor;    
    private Color highlightColor; 
    private Font noticeFont;
   
    /**
     * Constructs the custom dialog.
     * @param frame     the fame on the dialog will be displayed.
     * @param message   the message to display by the dialog.
     */    
    public NoticeDialog(JFrame frame, String message) {
        super(frame, true);
        loadColorFontTheme();
        setUndecorated(true);
            
	JPanel pane = new JPanel(new GridLayout(2, 0));
        pane.setBorder(BorderFactory.createLineBorder(foregroundColor,10));
               
        JLabel messageLabel = new JLabel(message);
        messageLabel.setFont(noticeFont); 
        pane.add(messageLabel);
        
        /* creates the back button */
        CustomButton button = new CustomButton("OK");
	button.addFocusListener(this);
	button.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {    /* when action ocurs */
                /* shows the main menu */
		dispose();
	    }
	});
        pane.add(button);        
        
        setContentPane(pane);
    }

    /**
     * Loads the colors and fonts from the config file.
     */       
    private void loadColorFontTheme() {
        ConfigReader config = new ConfigReader();
        foregroundColor = config.loadColor("foreground");        
        highlightColor = config.loadColor("highlight"); 
        noticeFont = config.loadFont("button");
    }          

    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* highlights the current button */
        e.getComponent().setForeground(highlightColor);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* restore the default color */
        e.getComponent().setForeground(foregroundColor);
    }    
}
