/*
 * NoticeDialog.java
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.gui.util;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.net.*;

import lhmc.utils.*;
import lhmc.network.*;

/**
 * Shared filessystems and removible devices choose dialog.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 20 Jun 2007
 */
public class SharedFileSystemsDialog extends JDialog implements FocusListener, ActionListener {
    
    /* Font Types and Colors */
    private Color foregroundColor;    
    private Color highlightColor; 
    private Font labelFont;
    
    private String remoteHost = null;
    private String hostType = null;
    
    
    /**
     * Constructs the chooser dialog.
     */    
    public SharedFileSystemsDialog(JFrame frame) {
        super(frame, true);
        loadColorFontTheme();
        setUndecorated(true);    
        
    }
        
    /**
     * Creates the panel of the chooser dialog.
     * @param filesystemLocation    the current location
     */     
    public void createHostsDialog(String filesystemLocation){
        
        JPanel dialogPanel = new JPanel();
        dialogPanel.setLayout(new BoxLayout(dialogPanel, BoxLayout.Y_AXIS));
        dialogPanel.setBorder(BorderFactory.createLineBorder(foregroundColor,10));        
        
        /* search available hosts */
        Vector<String> hosts = getHostsNames();
        hosts.add("Home");

        JLabel title = new JLabel("Change to: ");
        title.setFont(labelFont);        
        title.setAlignmentX(Component.CENTER_ALIGNMENT);                
        dialogPanel.add(title);
        
	JPanel pane = new JPanel(new GridLayout(hosts.size() + 2, 0));
      
        pane.add(Box.createRigidArea(new Dimension(0,3))); /* empty space */      
        for (int i = 0 ; i < hosts.size(); i++) {
              
                if (! (hosts.elementAt(i).equals(filesystemLocation))) {

                    /* creates a new button */
                    CustomButton button = new CustomButton();
                    button.setText(hosts.elementAt(i));
                    button.setFont(labelFont);  
                    button.addFocusListener(this);
                    button.addActionListener(this);		
                    if (i == 0) {
                        button.requestFocusInWindow(); 
                    }
                    pane.add(button);   /* adds button to the panel */
                }
        }
        
        /* creates the back button */
        CustomButton button = new CustomButton("Close");
        button.setFont(labelFont);  
        button.addFocusListener(this);
	button.addActionListener(this);
        pane.add(button);  
        
        dialogPanel.add(pane);
        
        setContentPane(dialogPanel);
    }    
    
    /**
     * Creates the panel of the chooser dialog to save a file.
     * @param filesystemLocation    the current location
     */       
    public void createToSaveDialog(String filesystemLocation){
        
        JPanel dialogPanel = new JPanel();
        dialogPanel.setLayout(new BoxLayout(dialogPanel, BoxLayout.Y_AXIS));
        dialogPanel.setBorder(BorderFactory.createLineBorder(foregroundColor,10));
        
        JLabel title = new JLabel("Save to: ");
        title.setFont(labelFont);           
        title.setAlignmentX(Component.CENTER_ALIGNMENT);        
        dialogPanel.add(title);
        
        // BUSCAR sistemes distriuits
        Vector<String> hosts = getHostsNames();
        hosts.add("Home");
        
        URL[] remDevices = getRemovableDevices();
        for (int i = 0; i < remDevices.length; i++ ) {          
            hosts.add(remDevices[i].getPath());
        }               
        
	JPanel pane = new JPanel(new GridLayout(hosts.size() + 2, 0));
       
	pane.add(Box.createRigidArea(new Dimension(0,3))); /* empty space */      
        for (int i = 0 ; i < hosts.size(); i++) {            
               if (! (hosts.elementAt(i).equals(filesystemLocation))) {

                    /* creates a new button */
                    CustomButton button = new CustomButton();
                    button.setText(hosts.elementAt(i));
                    
                    button.setFont(labelFont);  
                    
                    button.addFocusListener(this);
                    button.addActionListener(this);		
                    if (i == 0) {
                        button.requestFocusInWindow(); 
                    }
                    pane.add(button);   /* adds button to the panel */
                }
        }
            
        /* creates the back button */
        CustomButton button = new CustomButton("Close");
	button.addFocusListener(this);
	button.addActionListener(this);
        button.setFont(labelFont);          
        pane.add(button);        
        
        dialogPanel.add(pane);
        
        setContentPane(dialogPanel);        
    }
    
    /**
     * Gets an array of removible devices location
     * @return      an array of removible devices location.
     */
    public URL[] getRemovableDevices(){
        URL[] usbDevices = null;
        String pluginUrl = null;       
        URL root = null;
        ConfigReader config = new ConfigReader();        
        pluginUrl = config.getProperty("URLRemovableDev");
        try {
            root =  new URL("file://" + pluginUrl);
        } catch (MalformedURLException e) {
            System.out.println("MalformedURLException: " + e);   
        }
        RemovableDevicesActivator removableDevicesActivator = new RemovableDevicesActivator(root); 
        usbDevices = removableDevicesActivator.getRemovableDevices();           
        return usbDevices;                
    }

    /**
     * Gets the remote available hostnames.
     * @return      a vector of hostnames.
     */    
    public Vector<String> getHostsNames() {
        HostsVector hostsVector = new HostsVector();
        /* listen for hostName petitions */        
        Thread hostsSearch = new Thread(new HostsSearch(hostsVector));
        hostsSearch.start();         
        
        try {
            /* Pause for 500 microseconds */
            Thread.sleep(500);
        } catch (InterruptedException e) {
            System.out.println("InterruptedException: " + e);
        }                         
        return hostsVector.getHosts();
    }
    
    
    /**
     * Gets the choosed option.
     * @return      the name of the choosed option.
     */        
    public String getRemoteHost() {
        return remoteHost;
    }
    
    /**
     * Gets the choosed option type.
     * @return      the name of the choosed option type.
     */        
    public String getHostType() {
        return hostType;
    }    
    
    /**
     * Finds the corresponding type.
     * @param host  the choosed option.
     * @return      the name of the choosed option type.
     */      
    private String identifyType(String host) {        
        if (host == "Home") {
            return host;
        } else if (host == "Close") {
            return host;
        } else if (host.startsWith("/")) {
            return "Removable";
        } else {
            return "Remote";
        }
    }
    
    /**
     * Loads the colors and fonts from the config file.
     */        
    private void loadColorFontTheme() {
        ConfigReader config = new ConfigReader();
        foregroundColor = config.loadColor("foreground");        
        highlightColor = config.loadColor("highlight");   
        labelFont = config.loadFont("dialog");
    }      
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((JButton)e.getSource()).getText();   
        remoteHost = text;
        hostType = identifyType(text);
        hide();
    }

    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* highlights the current button */
        e.getComponent().setForeground(highlightColor);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* restore the default color */
        e.getComponent().setForeground(foregroundColor);
    }    
}
