/*
 * HostListener.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.network;

import java.io.*;
import java.net.*;

/**
 * HostListener is a threaded class that listens for a multicast messages
 * and when a multicast message arrive sends the hostname.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class HostListener implements Runnable {
    
    private static final int multicastPort = 4447;
    private static final int port = 4448;
    private static final String broadcastGroup = "230.0.0.1";    

    /**
     * Executes the HostListener thread.
     */    
    public void run() {
        try {
            
            /* creates multicast socket */
            InetAddress group = InetAddress.getByName(broadcastGroup);
            MulticastSocket multicastSocket = new MulticastSocket(multicastPort);
            multicastSocket.joinGroup(group);

            /* packet to receive*/
            byte[] buf = new byte[1000];
            DatagramPacket packet = new DatagramPacket(buf, buf.length);

            /* wait for search messages */
            while (true) {
                /* Wait for packet */
                multicastSocket.receive(packet); 
                String received = new String(packet.getData(), 0, packet.getLength());
                InetAddress serverAddress = packet.getAddress();
                Thread sendHostName = new Thread(new SendHostName(serverAddress, port));
                sendHostName.start();                
            }   
        } catch (IOException e) {
            e.printStackTrace();
        }                 
    }

    //public static void main(String[] args) throws IOException {
    //  (new Thread(new HostListener())).start();
    //}
        
}



