/*
 * HostListener.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.network;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * HostsSearch is a threaded class that sends a multicast message
 * and waits for ressponses. If there is a response saves the host name recived
 * to a HostsVector.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class HostsSearch implements Runnable {
    
    private static final int multicastPort = 4447;
    private static final int port = 4448;
    private static final String broadcastGroup = "230.0.0.1";    
    
    private static final String SEARCH_MESSAGE = "SEARCHING HOSTS";
    
    HostsVector hostsVector = null;
    
    /**
     * HostsSearch constructor.
     * @param hostsVector   the hostvector to fill.
     */        
    public HostsSearch(HostsVector hostsVector) {
        this.hostsVector = hostsVector;        
    }

    /**
     * Executes the HostsSearch thread.
     */        
    public void run() {
        
        /* Socket to recive the name of the hosts */
        ServerSocket serverSocket = null;        
        try {
            serverSocket = new ServerSocket(port);
            /* unlocks accept when timeout is over */            
            serverSocket.setSoTimeout(1000);
	} catch(IOException e) {
	    System.out.println( "IOException: " + e );
	}         
            
        /* search message */
        String message = SEARCH_MESSAGE;

	/* Multicast search message */
        try {
            InetAddress group = InetAddress.getByName(broadcastGroup);                   
            MulticastSocket multicastSocket = new MulticastSocket(multicastPort);
            multicastSocket.joinGroup(group);
            DatagramPacket packet = new DatagramPacket(message.getBytes(), message.length(), group, multicastPort);                                      
            multicastSocket.send(packet);            
            multicastSocket.leaveGroup(group);
        } catch(UnknownHostException e) {
	    System.out.println( "UnknownHostException: " + e );
	} catch(IOException e) {
	    System.out.println( "IOException: " + e );
	}  
        
        Socket socket = null;
         
                
        /* Receive responses with the hosts name, untill the thread ends */
        Thread t = new Thread(new WaitTimeout(2000));        
        t.start();
        while( t.isAlive() ) {
            try {
                socket = serverSocket.accept();
                /* for each accept create a thread  */
                Thread receiveHostName = new Thread(new ReceiveHostName(socket, hostsVector));
                receiveHostName.start();
            } catch(IOException e) {}
       
        }
        try {
            socket.close();       
            serverSocket.close();
        } catch(IOException e) {
                System.out.println( "IOException: " + e );
        }
    }

    //public static void main(String[] args) throws java.io.IOException {
    //    HostsVector hostsVector= new HostsVector();
    //    (new Thread(new HostsSearch(hostsVector))).start();      
    //}
    
    /**
     * WaitTimeout is an inner class thread that sleeps until timeout.
     * 
     * @author 	Cristina Roura Claver
     * @version 	0.01 Jun 2007
     */    
    private class WaitTimeout implements Runnable {
        int timeout = 0;
        
        /**
         * WaitTimeout constructor.
         * @param timeout   the number of miliseconds to wait.
         */          
        public WaitTimeout(int timeout) {
            this.timeout = timeout;
        }
        
        /**
        * Executes the WaitTimeout thread.
        */      
        public void run() {
            try {
                Thread.sleep(timeout);
            } catch (InterruptedException e) {}        
        }
    }                      
}
