/*
 * ReceiveHostName.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.network;

import java.io.*;
import java.net.*;

/**
 * ReceiveHostName is a threaded class that manages the received packet with the hostname.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class ReceiveHostName implements Runnable {
    
    Socket socket = null;        
    HostsVector hostsVector = null;
        
    /**
     * ReceiveHostName constructor.
     * @param socket        the socket that recives the packet.
     * @param hostsVector   the hostsVector to be filled. 
     */               
    public ReceiveHostName( Socket socket, HostsVector hostsVector ) {
        this.socket = socket;                      
        this.hostsVector = hostsVector;  
    }

    /**
     * Executes the ReceiveHostName thread.
     */       
    public void run() {
        try {
            DataInputStream input = new DataInputStream(socket.getInputStream());
            String hostName = input.readUTF();
            hostsVector.addHost(hostName);
            System.out.println("Remote said: \"" + hostName + "\"");  
        } catch(IOException e) {
            System.out.println( "IOException: " + e );
        }
    }
}