/*
 * SendHostName.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.network;

import java.io.*;
import java.net.*;

/**
 * SendHostName is a threaded class that sends a host name to the host that 
 * has made the petition.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class SendHostName implements Runnable {
    InetAddress serverAddress = null;
    int port = 0;

    /**
     * SendHostName constructor.
     * @param serverAddress     the address to send the new packet.
     * @param port              the port to send the new packet. 
     */             
    public SendHostName( InetAddress serverAddress, int port ) {
        this.serverAddress = serverAddress;
        this.port = port;
    }

    /**
     * Executes the SendHostName thread.
     */      
    public void run() {           
        /* create socket */
        Socket newSocket = null;
        try {  // handle bad host/port errors
            newSocket = new Socket(serverAddress.getHostName(), port);
        } catch (UnknownHostException e) {
            System.out.println("Unknown IP address for server?");
        } catch(IOException e) {
            System.out.println("No server found at specified port.");
	} catch (Exception e) {
            System.out.println("Something strange happened:" + e);
        }                
                
        try {
            DataOutputStream output = new DataOutputStream(newSocket.getOutputStream());                
            String hostName = InetAddress.getLocalHost().getHostName();
            output.writeUTF(hostName);
            newSocket.close();
        } catch (UnknownHostException e) {
            System.out.println("UnknownHostException: " + e);
        } catch (IOException e) {
            System.out.println("IOException: " + e);
        }
    }
}
