/*
 * BasicPlayerPlugin.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.io.File;
import java.io.PrintStream;
import java.io.*;
import java.util.Map;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.LayoutManager;
import java.awt.FlowLayout;
import java.awt.*;
import javax.swing.*;
import javax.swing.JPanel;
import java.awt.event.*;
import java.net.*;

import jcifs.smb.*;

import javazoom.jlgui.basicplayer.*;
import javazoom.jlgui.player.amp.playlist.*;

import lhmc.plugins.*;
import lhmc.files.*;

/**
 * BasicPlayerPlugin executes a single sound file or a directory of files making a playlist.
 * When a file is played a play panel is showed.
 * It supports MP3, OGG Vorbis, FLAC, MONKEY's AUDIO, WAV, AIFF, AU and SPEEX audio formats.
 * @version 	0.01 Mar 2007
 * @author 	Cristina Roura Claver
 */
public class BasicPlayerPlugin extends Plugin implements BasicPlayerListener, KeyListener, FocusListener {
      
    protected BasicPlayer player;    
    protected PlaylistItem currentItem = null;
    protected Map audioInfo = null;   
    protected float positonValue = 0.00f;
    protected double volume = 0.50;       
    protected boolean isPaused = false;
    protected boolean isMuted = false;    

    private static final Font titleFont = new Font("SansSerif", Font.BOLD, 40);
            
    private PrintStream out = null;
    private BasicController control;
    private PlaylistFactory plf = null;
    private Playlist playlist = null;    
    
    private JPanel playJPanel;
    private JPanel buttonsPanel;
    private JLabel songTitle;
    
    /* buttons */
    private JToggleButton skipBackwardButton = null;    
    private JToggleButton seekBackwardButton = null;
    private JToggleButton startButton = null;    
    private JToggleButton pauseButton = null;
    private JToggleButton stopButton = null;    
    private JToggleButton seekForwardButton = null;
    private JToggleButton skipForwardButton = null;  

    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;   
    
    private long fileLength = 0;
   
    UrlUtils urlUtils = null;    
    
    /**
     * Constructs the basic player plugin.
     */
    public BasicPlayerPlugin() {   
        out = System.out;
        
        /* creates the panel and the buttons */
        playJPanel = new JPanel();
        playJPanel.setLayout(new BoxLayout(playJPanel, BoxLayout.Y_AXIS));        
        buttonsPanel();
    }
    
    /**
     * Shows the play panel and plays a single song.
     * @param url  the song to play.
     */     
    public void play(URL url) {  
        /*
        jcifs.Config.registerSmbURLHandler();    
        String protocol = url.getProtocol();
        if (protocol.equals("file")) {
            File file = new File(url.getPath());
            songTitle = new JLabel("Playing: " + file.getName());
        } else if (protocol.equals("smb")) {
            try {
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());
                songTitle = new JLabel("Playing: " + smbFile.getName());
            } catch(MalformedURLException e) {
                System.out.println("MalformedURLException " + e);
            }
        }
        */
        urlUtils = new UrlUtils(url);
        if (urlUtils.getLocation() == LOCAL_FILE) {
            File file = new File(url.getPath());
            songTitle = new JLabel("Playing: " + file.getName());
        } else if (urlUtils.getLocation() == REMOTE_SMBFILE) {
            try {
                SmbFile smbFile = new SmbFile("smb:/" + url.getPath());
                songTitle = new JLabel("Playing: " + smbFile.getName());
            } catch(MalformedURLException e) {
                System.out.println("MalformedURLException " + e);
            }
        }        
        
        
        /* shows the play panel */
        //songTitle = new JLabel(file.getName());
        songTitle.setFont(titleFont);    
        songTitle.setAlignmentX(Component.CENTER_ALIGNMENT);        
        playJPanel.add(Box.createRigidArea(new Dimension(100,100))); /* empty space */                      
        playJPanel.add(songTitle);        
        playJPanel.add(Box.createRigidArea(new Dimension(500,500))); /* empty space */              
        playJPanel.add(buttonsPanel);            
        mainFrame.addToMainScreenCardLayout(playJPanel, "PLAY");        
        mainFrame.setMainScreenPanel("PLAY");
        stopButton.requestFocus();   
        
        /* play the song */
	run(url);
  }
    
    /**
     * Shows the play panel and creates a playlist for all the songs in the 
     * current directory, and plays one by one.
     * @param file  the songs directory to play.
     */
    
    public void playAll(URL file) { 
        /*
        jcifs.Config.registerSmbURLHandler();    
        String protocol = file.getProtocol();
        if (protocol.equals("file")) {
            urlUtils = new UrlUtils(LOCAL_FILE);
        } else if (protocol.equals("smb")) {
           urlUtils = new UrlUtils(REMOTE_SMBFILE);
        }        
        */
        
        urlUtils = new UrlUtils(file);
     
        //* shows the play panel 
        songTitle = new JLabel("Playing: ");
        songTitle.setFont(titleFont);  
        songTitle.setAlignmentX(Component.CENTER_ALIGNMENT);        
        playJPanel.add(songTitle);
	playJPanel.add(Box.createRigidArea(new Dimension(0,20))); /* empty space */        
        playJPanel.add(buttonsPanel);            
        mainFrame.addToMainScreenCardLayout(playJPanel, "PLAY");        
        mainFrame.setMainScreenPanel("PLAY");
        stopButton.requestFocus();   
        
        //* creates the play list from all the songs in the current directory *
        plf = PlaylistFactory.getInstance();
        playlist = plf.getPlaylist();       
        PlaylistItem pli;
                
        AudioFileFilter audioFileFilter = new AudioFileFilter();
        URL[] files = urlUtils.getFilteredUrls(file, audioFileFilter);
        FileManager fileManager = new FileManager(files);          
        int numSongs = fileManager.getNodesNumber();
       
        for (int i = 0; i < numSongs; i++) {
            if (!fileManager.isDirectoryNode(i)) {       
                
                URL song = fileManager.getFileAt(i);     
                pli = new PlaylistItem(urlUtils.getUrlName(song),urlUtils.getAbsolutePath(song),-1,true);
                playlist.appendItem(pli);
                
            }            
        }
        
        //* plays the first song in the playlist 
        playSongPlaylist(true);                  
  
      }
  
        
    /**
     * Starts to play a song.
     * @param urlFile  the song file.
     */       
    public void run(URL urlFile) {
	/* Instantiate BasicPlayer. */
	player = new BasicPlayer();
	
	/* BasicPlayer is a BasicController. */
	control = (BasicController) player;

	/* Register BasicPlayerTest to BasicPlayerListener events.
         * It means that this object will be notified on BasicPlayer
         * events such as : opened(...), progress(...), stateUpdated(...) 
         */
        player.addBasicPlayerListener(this);                                
        
	try {			

            jcifs.Config.registerSmbURLHandler();
            String protocol = urlFile.getProtocol();    
            InputStream inputStream = null;
            if (protocol.equals("file")) {   
                File    file = new File( urlFile.getPath() );   
                fileLength = file.length();                                
                inputStream= new BufferedInputStream( new FileInputStream( file ));
            } else if (protocol.equals("smb")) {
                SmbFile smbFile = new SmbFile("smb:/" + urlFile.getPath());   
                fileLength = smbFile.length();                                
                inputStream= new BufferedInputStream( new SmbFileInputStream( smbFile ));
            }        
           
            /* Open file, or URL or Stream (shoutcast) to play. */
	    control.open(inputStream);                                    

	    /* Start playback in a thread. */
	    control.play();
            
	    /* Set Volume (0 to 1.0). */
	    control.setGain(volume);
			
	    /* Set Pan (-1.0 to 1.0). */
	    control.setPan(0.0);

	} catch (BasicPlayerException e) {
	    e.printStackTrace();
	} catch (FileNotFoundException e) {
	    e.printStackTrace();
	} catch (MalformedURLException e) {
	    e.printStackTrace();
	} catch (SmbException e) {
	    e.printStackTrace();
	} catch (UnknownHostException e) {
            e.printStackTrace();
        }
    }
		
    /**
     * Play a song from the playlist.
     * @param nextSong  if true plays the next song, else plays the previous song.
     */           
    private void playSongPlaylist(boolean nextSong) {
        try {
            if (nextSong) {
                currentItem = nextSong();
            }
            else {
                currentItem = previousSong();
            }
            /* if there is a next or previous song */
            if (currentItem != null) {            
                URL song = urlUtils.createURL(currentItem.getLocation());
                System.out.println("URLPLAYLIST: " + currentItem.getLocation());
                /* sets the title of the current song in the panel */
                songTitle.setText("Playing: " + urlUtils.getUrlName(song));
                /* plays the song */
                run(song);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }        
    }
    
    /**
     * Ask the next song from playlist.
     * @return      the song to play.
     */
    private PlaylistItem nextSong() {
        playlist.nextCursor();
        PlaylistItem pli = playlist.getCursor();
        return pli;
    }

    /**
     * Ask the previous song from playlist.
     * @return      the song to play.
     */    
    private PlaylistItem previousSong() {
        playlist.previousCursor();
        PlaylistItem pli = playlist.getCursor();
        return pli;        
    }
    
    /**
     * Stops the current song and returns to the previous screen
     */
    private void quitAction() {
        try {	
            if (player != null) {
                control.stop();
            }
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }            
        mainFrame.restoreMainScreen(playJPanel);          
    }

    /**
     * Stops the current song.
     */    
    private void stopAction() {
        try {	
            control.stop();
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }         
    }

    /**
     * Pauses or resumes the current song.
     */
    private void pauseResumeAction() {
        if (isPaused) {
            try {                   
                control.resume();
                isPaused = false;
            } catch(BasicPlayerException be) {
                be.printStackTrace();
            }            
        } else {
            try {	                
                control.pause();
                isPaused = true;
            } catch(BasicPlayerException be) {
                be.printStackTrace();
            }                  
        }
    }

    /**
     * Plays the current song if it is stoped.
     */    
    private void playAction() {
        try {	              
            control.play();
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }          
    }
    
    /**
     * Seeks forward the current song.
     */     
    private void seekForwardAction() {
        try {
            positonValue = positonValue + 0.1f;
            /* Do not seek to a position too near from the end : it can cause freeze. MAX=98% */
            if (positonValue > 0.98f) {
   		positonValue = 0.98f;
            }                
            if (audioInfo.containsKey("audio.type")) {
                String type = (String) audioInfo.get("audio.type");
                /* Seek support for MP3 and WAV */ //(WAVE, AU, AIFF).
                if (((type.equalsIgnoreCase("mp3")) || (type.equalsIgnoreCase("wave"))) && (audioInfo.containsKey("audio.length.bytes"))) {	
                    long skipBytes = (long) Math.round(((Integer) audioInfo.get("audio.length.bytes")).intValue()*positonValue);     
                    control.seek(skipBytes);                    
                }
            }
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }                 
    }
    
    /**
     * Seeks backward the current song.
     */
    private void seekBackwardAction() {
        try {
            positonValue = positonValue - 0.1f;
            /* Do not seek to a position too near from the end : it can cause freeze. MAX=98% */
            if (positonValue > 0.98f) {
   		positonValue = 0.98f;
            }                
            if (audioInfo.containsKey("audio.type")) {
                String type = (String) audioInfo.get("audio.type");
                /* Seek support for MP3 and WAV */ //(WAVE, AU, AIFF).
                if (((type.equalsIgnoreCase("mp3")) || (type.equalsIgnoreCase("wave"))) && (audioInfo.containsKey("audio.length.bytes"))) {	
                    long skipBytes = (long) Math.round(((Integer) audioInfo.get("audio.length.bytes")).intValue()*positonValue);     
                    control.seek(skipBytes);                    
                }              
            }                
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }          
    }
    
    /**
     * Increments the volume.
     */       
    private void volumeUpAction() {
        if (volume < 1.0) {
            volume = volume + 0.05;
        }
        try {                
            control.setGain(volume);
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }   
    }
    
    /**
     * Decrements the volume.
     */       
    private void volumeDownAction() {
        if (volume > 0.0) {
            volume = volume - 0.05;
        }
        try {	
            control.setGain(volume);
        } catch(BasicPlayerException be) {
            be.printStackTrace();
        }          
    }
    
    /**
     * Mutes or unmutes the volume.
     */     
    private void muteAction() {
        if (isMuted) {            
            try {	
                isMuted = false;                    
                control.setGain(volume);
            } catch(BasicPlayerException be) {
                be.printStackTrace();
            }            
        } else {
            try {	
                isMuted = true;                        
                control.setGain(0.0);
            } catch(BasicPlayerException be) {
                be.printStackTrace();
            }                 
        }        
    }

    /**
     * Stops the current song and plays the next in the playlist.
     */      
    private void skipForwardAction() {
        stopAction();
        playSongPlaylist(true);
    }
    
    /**
     * Stops the current song and plays the previous in the playlist.
     */          
    private void skipBackwardAction() {
        stopAction();
        playSongPlaylist(false);        
    }
            
    /**
     * Creates the buttons panel.
     */       
    private void buttonsPanel() {
        buttonsPanel = new JPanel(new BorderLayout());
        buttonsPanel.setLayout(new FlowLayout());
                 
        /* Skip-backward button */
        skipBackwardButton = new JToggleButton();
        skipBackwardButton.setIcon(new ImageIcon("images/skip-backward.png"));
        skipBackwardButton.setText("skip-backward");        
        skipBackwardButton.setBorderPainted(false);
        skipBackwardButton.setFocusable(true);
        skipBackwardButton.setFocusPainted(false);
        skipBackwardButton.addFocusListener(this);
        skipBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipBackwardButton);   
    
        /* seek-backward button */
        seekBackwardButton = new JToggleButton();
        seekBackwardButton.setIcon(new ImageIcon("images/seek-backward.png"));
        seekBackwardButton.setText("seek-backward");        
        seekBackwardButton.setBorderPainted(false);
        seekBackwardButton.setFocusable(true);
        seekBackwardButton.setFocusPainted(false);
        seekBackwardButton.addFocusListener(this);
        seekBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekBackwardButton);    

        /* start button */
        startButton = new JToggleButton();
        startButton.setIcon(new ImageIcon("images/start.png"));
        startButton.setText("start");        
        startButton.setBorderPainted(false);
        startButton.setFocusable(true);
        startButton.setFocusPainted(false);
        startButton.addFocusListener(this);
        startButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(startButton);           
        
        /* pause button */
        pauseButton = new JToggleButton();
        pauseButton.setIcon(new ImageIcon("images/pause.png"));
        pauseButton.setText("pause");        
        pauseButton.setBorderPainted(false);
        pauseButton.setFocusable(true);
        pauseButton.setFocusPainted(false);
        pauseButton.addFocusListener(this);
        pauseButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(pauseButton);      

        /* stop button */
        stopButton = new JToggleButton();
        stopButton.setIcon(new ImageIcon("images/stop.png"));
        stopButton.setText("stop");        
        stopButton.setBorderPainted(false);
        stopButton.setFocusable(true);
        stopButton.setFocusPainted(false);
        stopButton.addFocusListener(this);
        stopButton.addKeyListener(this);
        stopButton.requestFocusInWindow();         
        /* adds button to the panel */
        buttonsPanel.add(stopButton);        
           
        /* seek-forward. button */
        seekForwardButton = new JToggleButton();
        seekForwardButton.setIcon(new ImageIcon("images/seek-forward.png"));
        seekForwardButton.setText("seek-forward");        
        seekForwardButton.setBorderPainted(false);
        seekForwardButton.setFocusable(true);
        seekForwardButton.setFocusPainted(false);
        seekForwardButton.addFocusListener(this);
        seekForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekForwardButton);  

        /* skip-forward button */
        skipForwardButton = new JToggleButton();
        skipForwardButton.setIcon(new ImageIcon("images/skip-forward.png"));
        skipForwardButton.setText("skip-forward");        
        skipForwardButton.setBorderPainted(false);
        skipForwardButton.setFocusable(true);
        skipForwardButton.setFocusPainted(false);
        skipForwardButton.addFocusListener(this);
        skipForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipForwardButton);         
    }    
     
    /**
     * Open callback, stream is ready to play.
     *
     * properties map includes audio format dependant features such as
     * bitrate, duration, frequency, channels, number of frames, vbr flag,
     * id3v2/id3v1 (for MP3 only), comments (for Ogg Vorbis), ... 
     *
     * @param stream could be File, URL or InputStream
     * @param properties audio stream properties.
     */
    public void opened(Object stream, Map properties) {
	/* 
         * Pay attention to properties. It's useful to get duration, 
         * bitrate, channels, even tag such as ID3v2.
         */
        audioInfo = properties;
	//display("opened : " + properties.toString());		
    }
		
    /**
     * Progress callback while playing.
     * 
     * This method is called severals time per seconds while playing.
     * properties map includes audio format features such as
     * instant bitrate, microseconds position, current frame number, ... 
     * 
     * @param bytesread from encoded stream.
     * @param microseconds elapsed (<b>reseted after a seek !</b>).
     * @param pcmdata PCM samples.
     * @param properties audio stream parameters.
     */
    public void progress(int bytesread, long microseconds, byte[] pcmdata, Map properties) {
	/* 
         * Pay attention to properties. It depends on underlying JavaSound SPI
         * MP3SPI provides mp3.equalizer.
         */
	//display("progress : " + properties.toString());
    }
	
    /**
     * Notification callback for basicplayer events such as opened, eom ...
     *  
     * @param event
     */
    public void stateUpdated(BasicPlayerEvent event) {
	/*
         * Notification of BasicPlayer states (opened, playing, end of media, ...)
         * display("stateUpdated : " + event.toString());
	 * End of song reached ? So play next song.
         */
        if (event.getCode() == BasicPlayerEvent.EOM && (currentItem != null) ) {
	    playSongPlaylist(true);
	}     
        if (event.getCode() == BasicPlayerEvent.EOM && (currentItem == null) ) {
	    quitAction();
	}         
    }

    /**
     * A handle to the BasicPlayer, plugins may control the player through
     * the controller (play, stop, ...)
     * @param controller : a handle to the player
     */	
    public void setController(BasicController controller) {
	display("setController : " + controller);
    }

    /**
     * Displays a message.
     * @param   msg     A string of the message.
     */	    
    public void display(String msg) {
	if (out != null) out.println(msg);
    }    
    
    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* paint border the current button */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(true);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* erase the border */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(false);
    }     
    
    /*
     * Invoked when a key is pressed.
     * @param e The key event
     */       
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        /* make sure the key isn't processed for anything else */
        e.consume();           
        if (key == KeyEvent.VK_ESCAPE) {            /* QUIT */   
            quitAction();
        } else if (key == KeyEvent.VK_S) {          /* STOP */   
            stopButton.requestFocus();
            stopButton.doClick();
            stopAction();
            stopButton.doClick();
        } else if (key == KeyEvent.VK_L) {         /* PAUSE - RESUME */       
            pauseButton.requestFocus();
            pauseButton.doClick();
            pauseResumeAction();
        } else if (key == KeyEvent.VK_P) {         /* PLAY */       
            startButton.requestFocus();
            startButton.doClick();
            playAction();
            startButton.doClick();
        } else if (key == KeyEvent.VK_C) {         /* SEEK + */     
            seekForwardButton.requestFocus();
            seekForwardButton.doClick();
            seekForwardAction();
            seekForwardButton.doClick();
        } else if (key == KeyEvent.VK_X) {         /* SEEK - */   
            seekBackwardButton.requestFocus();
            seekBackwardButton.doClick();
            seekBackwardAction();
            seekBackwardButton.doClick();
        } else if (key == KeyEvent.VK_U) {         /* VOL UP */    
            volumeUpAction();
        } else if (key == KeyEvent.VK_I) {         /* VOL DOWN */       
           volumeDownAction();          
        } else if (key == KeyEvent.VK_M) {         /* MUTE */       
            muteAction();
        } else if (key == KeyEvent.VK_V) {         /* NEXT */     
            skipForwardButton.requestFocus();
            skipForwardButton.doClick();
            skipForwardAction();
            skipForwardButton.doClick();
        }  else if (key == KeyEvent.VK_W) {         /* PREVIOUS */     
            skipBackwardButton.requestFocus();
            skipBackwardButton.doClick();
            skipBackwardAction();
            skipBackwardButton.doClick();
        } else if (key == KeyEvent.VK_ENTER) {      /* ACTION KEY */
            JToggleButton button = (JToggleButton) e.getSource();
            String text =button.getText();   
            System.out.println("ACTION BUTON TEXT: " + text);
            if (text == "skip-backward") {
                button.doClick();
                skipBackwardAction();
                button.doClick();
            } else if (text == "seek-backward") {
                button.doClick();
                seekBackwardAction();
                button.doClick();
            } else if (text == "start") {
                button.doClick();
                playAction();
                button.doClick();
            } else if (text == "pause") {
                button.doClick();
                pauseResumeAction();                
            } else if (text == "stop") {
                button.doClick();
                stopAction();                
                button.doClick();
            } else if (text == "seek-forward") {
                button.doClick();
                seekForwardAction();
                button.doClick();
            } else if (text == "skip-forward") {
                button.doClick();
                skipForwardAction();
                button.doClick();
            }
        }
    }

    /*
     * Invoked when a key is released.
     * @param e The key event
     */       
    public void keyReleased(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }
    
    /*
     * Invoked when a key is typed.
     * @param e The key event
     */       
    public void keyTyped(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }      


}