/*
 * CddaMPlayerPlugin.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.io.File;
import java.io.IOException;
import javax.imageio.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.BorderLayout;
import java.awt.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import java.io.PrintStream;
import java.io.*;
import java.util.Map;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.LayoutManager;
import java.awt.FlowLayout;
import java.awt.*;
import javax.swing.*;
import javax.swing.JPanel;
import java.awt.event.*;
import java.net.*;

import lhmc.plugins.*;
import lhmc.gui.util.*;


/**
 * CddaMPlayerPlugin plays an audio CD with MPlayer
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Mar 2007
 */
public class CddaMPlayerPlugin extends Plugin implements KeyListener, FocusListener {

    /* fullscreen */
    protected GraphicsDevice device;
    protected Window mainWindow;    
        
    /* needed to avoid serialization warnings */
    private static final long serialVersionUID = 1L;
    
    private static final Font titleFont = new Font("SansSerif", Font.BOLD, 40);    
    
    private JPanel playJPanel;
    private JPanel buttonsPanel;
    private JLabel songTitle;
    
    /* buttons */
    private JToggleButton skipBackwardButton = null;    
    private JToggleButton seekBackwardButton = null;
    private JToggleButton startButton = null;    
    private JToggleButton pauseButton = null;
    private JToggleButton stopButton = null;    
    private JToggleButton seekForwardButton = null;
    private JToggleButton skipForwardButton = null;      
    
    private Thread runner = null;

    private Process playerPrc;  
    
    static BufferedReader stdout, stderr;
    static Vector<String> outputLines;
    private boolean paused = false;
    private boolean outputAllowed = false;
    
    private float volume = 50;       //0-100
    private boolean isMuted = false;      
    private int currentNumTrack = 1;
    private int maxNumTracks = 0;
    
    private ExternalPlugin ep;
    
    private boolean isPlaying = false;
    
    /**
     * Constructs CddaMPlayerPlugin.
     */
    CddaMPlayerPlugin() {   
        try {
                /* find out number of tracks */
                String[] command = new String[] { "cdir" };
                Process process = Runtime.getRuntime().exec(command);
                InputStream is = process.getInputStream();

                BufferedReader stdout = new BufferedReader(new InputStreamReader(is));
                String line;
                int numTracks = -1;
                while(( line = stdout.readLine()) != null) {                         
                    System.out.println(line);
                    numTracks++;
                }
                System.out.println("Number of tracks: " + numTracks);
                maxNumTracks = numTracks;
        } catch (IOException ie) {
            System.out.println("IO exception on stdout: " + ie);
        }            
        
        /* creates the panel and the buttons */
        playJPanel = new JPanel(); 
        buttonsPanel();                     
    }
    
    /**
     * Shows the play panel and plays the first track of an audio Cd.
     * @param file  not in use.
     */    
    public void play(URL file) {        
        
        /* shows the play panel */  
        songTitle = new JLabel("");
        songTitle.setFont(titleFont);    
        playJPanel.add(songTitle, BorderLayout.CENTER);
        playJPanel.add(buttonsPanel, BorderLayout.SOUTH);            
        mainFrame.addToMainScreenCardLayout(playJPanel, "PLAY");        
        mainFrame.setMainScreenPanel("PLAY");
        stopButton.requestFocus();   
        
        
        /* play the song */
	String[] command = generateCommand();
        
        songTitle.setText("Track Number: " + currentNumTrack);		
	ep = new ExternalPlugin(command);
        playerPrc = ep.executeCommand();
        ep.start();  
        isPlaying = true;
    }

    /**
     * Creates the command to be executed.
     * @return      the command string.
     */      
    public String[] generateCommand() {
	String[] command = new String[] {
	    "/usr/bin/mplayer",    
            "-slave", /* Instead of intercepting keyboard events, 
                        MPlayer will read commands separated by a newline (\n) from stdin */
	    "cdda://"+currentNumTrack
	};        
        return command;
    }

    public void playAll(URL file) {
    }

    /**
     * Stops the current song and returns to the previous screen
     */
    private void quitAction() {
        PrintStream s = new PrintStream(playerPrc.getOutputStream());
        s.print("quit\n");
        s.flush();        
        ep.stop();  
        isPlaying = false;
        mainFrame.restoreMainScreen(playJPanel);          
    }

    /**
     * Stops the current song.
     */    
    private void stopAction() {
        if (isPlaying) {
            ep.stop();
            isPlaying = false;
            try {
                //Pause for 200 microseconds
                Thread.sleep(200);
            } catch (InterruptedException e) {
                System.out.println("InterruptedException: " + e);
            }          
        }
    }

    /**
     * Pauses or resumes the current song.
     */
    private void pauseResumeAction() {
        PrintStream s = new PrintStream(playerPrc.getOutputStream());
        s.print("pause\n");
        s.flush();  
    }

    /**
     * Plays the current song if it is stoped.
     */    
    private void playAction() {        
        if (!isPlaying) {
            ep.setCommand(generateCommand());
            playerPrc = ep.executeCommand();
            ep.start();
            isPlaying = true;
        }
    }
    
    /**
     * Seeks forward the current song.
     */     
    private void seekForwardAction() {
        PrintStream s = new PrintStream(playerPrc.getOutputStream());               
        String command = "seek +30 0\n";        
        s.print(command);
        s.flush();               
    }
    
    /**
     * Seeks backward the current song.
     */       
    private void seekBackwardAction() {
        PrintStream s = new PrintStream(playerPrc.getOutputStream());
        String command = "seek -30 0\n";        
        s.print(command);        
        s.flush();       
    }
    

    /**
     * Stops the current song and plays the next in the playlist.
     */      
    private void skipForwardAction() {
        stopAction();      
        if (currentNumTrack < maxNumTracks) {
            currentNumTrack++;                  
            playAction();
            songTitle.setText("Track Number: " + currentNumTrack);
        } else {                     
            songTitle.setText("Track Number: " + currentNumTrack);
        }
    }
    
    /**
     * Stops the current song and plays the previous in the playlist.
     */          
    private void skipBackwardAction() {
        stopAction();     
        if (currentNumTrack > 1) {        
            currentNumTrack--;               
            playAction();
            songTitle.setText("Track Number: " + currentNumTrack);            
        } else {   
            songTitle.setText("Track Number: " + currentNumTrack);
        }
    }
    
    
    /**
     * Increments the volume.
     */       
    private void volumeUpAction() {
        if (volume < 100) {
            volume = volume + 5;
        }
        PrintStream s = new PrintStream(playerPrc.getOutputStream());
        s.print("volume " + volume + " \n");
        s.flush(); 
    }
    
    /**
     * Decrements the volume.
     */       
    private void volumeDownAction() {
        if (volume > 0) {
            volume = volume - 5;
        }
        PrintStream s = new PrintStream(playerPrc.getOutputStream());
        s.print("volume " + volume + " \n");
        s.flush();          
    }
    
    /**
     * Mutes or unmutes the volume.
     */     
    private void muteAction() {
        PrintStream s = new PrintStream(playerPrc.getOutputStream());        
        if (isMuted) {            
            s.print("mute 0\n");
            isMuted = false;                            
        } else {
            s.print("mute 1\n");
            isMuted = true;
        }                
        s.flush();          
    }
    
    /**
     * Creates the buttons panel.
     */       
    private void buttonsPanel() {
        buttonsPanel = new JPanel(new BorderLayout());
        buttonsPanel.setLayout(new FlowLayout());
                 
        /* Skip-backward button */
        skipBackwardButton = new JToggleButton();
        skipBackwardButton.setIcon(new ImageIcon("images/skip-backward.png"));
        skipBackwardButton.setText("skip-backward");        
        skipBackwardButton.setBorderPainted(false);
        skipBackwardButton.setFocusable(true);
        skipBackwardButton.setFocusPainted(false);
        skipBackwardButton.addFocusListener(this);
        skipBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipBackwardButton);   
    
        /* seek-backward button */
        seekBackwardButton = new JToggleButton();
        seekBackwardButton.setIcon(new ImageIcon("images/seek-backward.png"));
        seekBackwardButton.setText("seek-backward");        
        seekBackwardButton.setBorderPainted(false);
        seekBackwardButton.setFocusable(true);
        seekBackwardButton.setFocusPainted(false);
        seekBackwardButton.addFocusListener(this);
        seekBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekBackwardButton);    

        /* start button */
        startButton = new JToggleButton();
        startButton.setIcon(new ImageIcon("images/start.png"));
        startButton.setText("start");        
        startButton.setBorderPainted(false);
        startButton.setFocusable(true);
        startButton.setFocusPainted(false);
        startButton.addFocusListener(this);
        startButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(startButton);           
        
        /* pause button */
        pauseButton = new JToggleButton();
        pauseButton.setIcon(new ImageIcon("images/pause.png"));
        pauseButton.setText("pause");        
        pauseButton.setBorderPainted(false);
        pauseButton.setFocusable(true);
        pauseButton.setFocusPainted(false);
        pauseButton.addFocusListener(this);
        pauseButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(pauseButton);      

        /* stop button */
        stopButton = new JToggleButton();
        stopButton.setIcon(new ImageIcon("images/stop.png"));
        stopButton.setText("stop");        
        stopButton.setBorderPainted(false);
        stopButton.setFocusable(true);
        stopButton.setFocusPainted(false);
        stopButton.addFocusListener(this);
        stopButton.addKeyListener(this);
        stopButton.requestFocusInWindow();         
        /* adds button to the panel */
        buttonsPanel.add(stopButton);        
           
        /* seek-forward. button */
        seekForwardButton = new JToggleButton();
        seekForwardButton.setIcon(new ImageIcon("images/seek-forward.png"));
        seekForwardButton.setText("seek-forward");        
        seekForwardButton.setBorderPainted(false);
        seekForwardButton.setFocusable(true);
        seekForwardButton.setFocusPainted(false);
        seekForwardButton.addFocusListener(this);
        seekForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekForwardButton);  

        /* skip-forward button */
        skipForwardButton = new JToggleButton();
        skipForwardButton.setIcon(new ImageIcon("images/skip-forward.png"));
        skipForwardButton.setText("skip-forward");        
        skipForwardButton.setBorderPainted(false);
        skipForwardButton.setFocusable(true);
        skipForwardButton.setFocusPainted(false);
        skipForwardButton.addFocusListener(this);
        skipForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipForwardButton);         
    }        

    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* paint border the current button */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(true);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* erase the border */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(false);
    }     
    
    /*
     * Invoked when a key is pressed.
     * @param e The key event
     */       
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        /* make sure the key isn't processed for anything else */
        e.consume();           
        if (key == KeyEvent.VK_ESCAPE) {            /* QUIT */   
            quitAction();
        } else if (key == KeyEvent.VK_S) {          /* STOP */   
            stopButton.requestFocus();
            stopButton.doClick();
            stopAction();
            stopButton.doClick();
        } else if (key == KeyEvent.VK_L) {         /* PAUSE - RESUME */       
            pauseButton.requestFocus();
            pauseButton.doClick();
            pauseResumeAction();
        } else if (key == KeyEvent.VK_P) {         /* PLAY */       
            startButton.requestFocus();
            startButton.doClick();
            playAction();
            startButton.doClick();
        } else if (key == KeyEvent.VK_C) {         /* SEEK + */     
            seekForwardButton.requestFocus();
            seekForwardButton.doClick();
            seekForwardAction();
            seekForwardButton.doClick();
        } else if (key == KeyEvent.VK_X) {         /* SEEK - */   
            seekBackwardButton.requestFocus();
            seekBackwardButton.doClick();
            seekBackwardAction();
            seekBackwardButton.doClick();
        } else if (key == KeyEvent.VK_U) {         /* VOL UP */    
            volumeUpAction();
        } else if (key == KeyEvent.VK_I) {         /* VOL DOWN */       
           volumeDownAction();          
        } else if (key == KeyEvent.VK_M) {         /* MUTE */       
            muteAction();
        } else if (key == KeyEvent.VK_V) {         /* NEXT */     
            skipForwardButton.requestFocus();
            skipForwardButton.doClick();
            skipForwardAction();
            skipForwardButton.doClick();
        }  else if (key == KeyEvent.VK_W) {         /* PREVIOUS */     
            skipBackwardButton.requestFocus();
            skipBackwardButton.doClick();
            skipBackwardAction();
            skipBackwardButton.doClick();            
        } else if (key == KeyEvent.VK_ENTER) {      /* ACTION KEY */
            JToggleButton button = (JToggleButton) e.getSource();
            String text =button.getText();   
            System.out.println("ACTION BUTON TEXT: " + text);
            if (text == "skip-backward") {
                button.doClick();
                skipBackwardAction();
                button.doClick();
            } else if (text == "seek-backward") {
                button.doClick();
                seekBackwardAction();
                button.doClick();
            } else if (text == "start") {
                button.doClick();
                playAction();
                button.doClick();
            } else if (text == "pause") {
                button.doClick();
                pauseResumeAction();                
            } else if (text == "stop") {
                button.doClick();
                stopAction();                
                button.doClick();
            } else if (text == "seek-forward") {
                button.doClick();
                seekForwardAction();
                button.doClick();
            } else if (text == "skip-forward") {
                button.doClick();
                skipForwardAction();
                button.doClick();
            }
        }
    }

    /*
     * Invoked when a key is released.
     * @param e The key event
     */       
    public void keyReleased(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }
    
    /*
     * Invoked when a key is typed.
     * @param e The key event
     */       
    public void keyTyped(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }     
}