/*
 *	CddaPlayerApplication.java
 *
 *	This file is part of jsresources.org
 */

/*
 * Copyright (c) 2001 by Matthias Pfisterer
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * CddaPlayerPlugin.java	
 *
 * Modified by: Cristina Roura Claver cristina.roura@gmail.com
 */
package lhmc.plugins;

import java.io.File;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import java.net.URL;
import java.net.MalformedURLException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineUnavailableException;

import java.io.PrintStream;
import java.io.*;
import java.util.Map;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.LayoutManager;
import java.awt.FlowLayout;
import java.awt.*;
import javax.swing.*;
import javax.swing.JPanel;
import java.awt.event.*;

import lhmc.plugins.*;
import lhmc.gui.util.*;

import org.tritonus.sampled.cdda.CddaURLStreamHandlerFactory;

/**
 * CddaMPlayerPlugin plays an audio CD
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Mar 2007
 */
public class CddaPlayerPlugin extends Plugin implements KeyListener, FocusListener {
    static {
	URL.setURLStreamHandlerFactory(new CddaURLStreamHandlerFactory());
    }

    /*
     * Flag for debugging messages.
     * If true, some messages are dumped to the console
     * during operation.	
     */
    private static boolean	DEBUG = true;

    private static int		DEFAULT_EXTERNAL_BUFFER_SIZE = 128000;
    
    private float volume = 50;       //0-100
    private boolean isMuted = false;    
    private int currentNumTrack = 1;
    private int maxNumTracks = 0;
    
    private static final Font titleFont = new Font("SansSerif", Font.BOLD, 40);    
    
    private JPanel playJPanel;
    private JPanel buttonsPanel;
    private JLabel songTitle;
    
    /* buttons */
    private JToggleButton skipBackwardButton = null;    
    private JToggleButton seekBackwardButton = null;
    private JToggleButton startButton = null;    
    private JToggleButton pauseButton = null;
    private JToggleButton stopButton = null;    
    private JToggleButton seekForwardButton = null;
    private JToggleButton skipForwardButton = null;       
    
    private boolean isPlaying = false;
    SourceDataLine	line = null;

    /**
     * Constructs CddaPlayerPlugin.
     */
    public CddaPlayerPlugin() {
           try {
                /* find out number of tracks */
                String[] command = new String[] { "cdir" };
                Process process = Runtime.getRuntime().exec(command);
                InputStream is = process.getInputStream();

                BufferedReader stdout = new BufferedReader(new InputStreamReader(is));
                String line;
                int numTracks = -1;
                while(( line = stdout.readLine()) != null) {                         
                    System.out.println(line);
                    numTracks++;
                }
                maxNumTracks = numTracks;
                 System.out.println("Number of tracks: " + numTracks);
        } catch (IOException ie) {
            System.out.println("IO exception on stdout: " + ie);
        }                 

        /* creates the panel and the buttons */
        playJPanel = new JPanel(); 
        buttonsPanel();             
    }
    
    public void playAll(URL file) {       
    }

    /**
     * Shows the play panel and plays the first track of an audio Cd.
     * @param file  not in use.
     */    
    public void play(URL file) {
        /* shows the play panel */  
        songTitle = new JLabel("");
        songTitle.setFont(titleFont);    
        playJPanel.add(songTitle, BorderLayout.CENTER);
        playJPanel.add(buttonsPanel, BorderLayout.SOUTH);            
        mainFrame.addToMainScreenCardLayout(playJPanel, "PLAY");        
        mainFrame.setMainScreenPanel("PLAY");
        stopButton.requestFocus();                   
        /* play the song */        
        playAction();
    }

    /**
     * Outputs and inputStrem.
     * @param inputStream the inputStream.
     */      
    private static void output(InputStream inputStream) {
	byte[]		buffer = new byte[4096];
	OutputStream	outputStream = System.out;

	try {
	    int	nBytesRead = 0;
	    nBytesRead = inputStream.read(buffer);
	    while (nBytesRead >= 0) {
		outputStream.write(buffer, 0, nBytesRead);
		nBytesRead = inputStream.read(buffer);
	    }
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }

    /**
     * Outputs a message.
     * @param strMessage the message.
     */    
    private static void out(String strMessage) {
	System.out.println(strMessage);
    }

    /**
     * Plays a track of an audio Cd.
     */       
    public void playAction() {
        if (!isPlaying) {
            isPlaying = true;
            int	nExternalBufferSize = DEFAULT_EXTERNAL_BUFFER_SIZE;
            int	nInternalBufferSize = AudioSystem.NOT_SPECIFIED;

            /** CDROM drive number.
             *  Defaults to first drive. [how is the order constituted?]
             *  Not used for now. Hardcoded default to /dev/cdrom.
             */
            int	nDrive = 0;
            boolean	bTocOnly = true;

            if (maxNumTracks == 0) {
                bTocOnly = true;
            } else if (maxNumTracks > 0) {
                bTocOnly = false;
            }

            // TODO: should not be hardcoded
            String strDrive = "/dev/cdrom";

            if (bTocOnly) {
                URL tocURL = null;
                try {
                    tocURL = new URL("cdda:" + strDrive);
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }

                InputStream tocInputStream = null;
                try {
                    tocInputStream = tocURL.openStream();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                output(tocInputStream);
            }

            if (! bTocOnly) {
                URL trackURL = null;
                try {
                    trackURL = new URL("cdda://" + strDrive + "#" + currentNumTrack);
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                }
                InputStream trackInputStream = null;
                try {
                    trackInputStream = trackURL.openStream();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                AudioInputStream 	audioInputStream = (AudioInputStream) trackInputStream;


                AudioFormat		audioFormat = audioInputStream.getFormat();
                Line.Info		info = new DataLine.Info(SourceDataLine.class, audioFormat);

                try {
                    line = (SourceDataLine) AudioSystem.getLine(info);
                    line.open();
                    line.start();
                } catch (LineUnavailableException e) {
                    e.printStackTrace();
                }

                int		nBytesRead = 0;
                byte[]	abData = new byte[nExternalBufferSize];

                while (nBytesRead != -1) {
                    try {
                        nBytesRead = audioInputStream.read(abData, 0, abData.length);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    if (DEBUG) {
                        out("AudioPlayer.main(): read from AudioInputStream (bytes): " + nBytesRead);
                    }
                    if (nBytesRead >= 0) {
                        int	nBytesWritten = line.write(abData, 0, nBytesRead);
                        if (DEBUG) {
                            out("AudioPlayer.main(): written to SourceDataLine (bytes): " 
                                    + nBytesWritten);
                        }
                    }
                }
            }      
        }
    }
    
    /**
     * Stops the current song and returns to the previous screen
     */
    private void quitAction() {
        line.stop();
        isPlaying = false;
        mainFrame.restoreMainScreen(playJPanel);          
    }

    /**
     * Stops the current song.
     */    
    private void stopAction() {
        if (isPlaying) {
            line.stop();
            isPlaying = false;
            try {
                //Pause for 200 microseconds
                Thread.sleep(200);
            } catch (InterruptedException e) {
                System.out.println("InterruptedException: " + e);
            }          
        }
    }

    /**
     * Pauses or resumes the current song.
     */
    private void pauseResumeAction() {
    }

    
    /**
     * Seeks forward the current song.
     */     
    private void seekForwardAction() {
           
    }
    
    /**
     * Seeks backward the current song.
     */       
    private void seekBackwardAction() {
    
    }
    

    /**
     * Stops the current song and plays the next in the playlist.
     */      
    private void skipForwardAction() {
        stopAction();      
        if (currentNumTrack < maxNumTracks) {
            currentNumTrack++;                  
            playAction();
            songTitle.setText("Track Number: " + currentNumTrack);
        } else {                     
            songTitle.setText("Track Number: " + currentNumTrack);
        }
    }
    
    /**
     * Stops the current song and plays the previous in the playlist.
     */          
    private void skipBackwardAction() {
        stopAction();     
        if (currentNumTrack > 1) {        
            currentNumTrack--;               
            playAction();
            songTitle.setText("Track Number: " + currentNumTrack);            
        } else {   
            songTitle.setText("Track Number: " + currentNumTrack);
        }
    }
    
    
    /**
     * Increments the volume.
     */       
    private void volumeUpAction() {
        if (volume < 100) {
            volume = volume + 5;
        }
    }
    
    /**
     * Decrements the volume.
     */       
    private void volumeDownAction() {
        if (volume > 0) {
            volume = volume - 5;
        }
      
    }
    
    /**
     * Mutes or unmutes the volume.
     */     
    private void muteAction() {

        if (isMuted) {            

            isMuted = false;                            
        } else {
    
            isMuted = true;
        }                
           
    }
    
    
    /**
     * Creates the buttons panel.
     */       
    private void buttonsPanel() {
        buttonsPanel = new JPanel(new BorderLayout());
        buttonsPanel.setLayout(new FlowLayout());
                 
        /* Skip-backward button */
        skipBackwardButton = new JToggleButton();
        skipBackwardButton.setIcon(new ImageIcon("images/skip-backward.png"));
        skipBackwardButton.setText("skip-backward");        
        skipBackwardButton.setBorderPainted(false);
        skipBackwardButton.setFocusable(true);
        skipBackwardButton.setFocusPainted(false);
        skipBackwardButton.addFocusListener(this);
        skipBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipBackwardButton);   
    
        /* seek-backward button */
        seekBackwardButton = new JToggleButton();
        seekBackwardButton.setIcon(new ImageIcon("images/seek-backward.png"));
        seekBackwardButton.setText("seek-backward");        
        seekBackwardButton.setBorderPainted(false);
        seekBackwardButton.setFocusable(true);
        seekBackwardButton.setFocusPainted(false);
        seekBackwardButton.addFocusListener(this);
        seekBackwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekBackwardButton);    

        /* start button */
        startButton = new JToggleButton();
        startButton.setIcon(new ImageIcon("images/start.png"));
        startButton.setText("start");        
        startButton.setBorderPainted(false);
        startButton.setFocusable(true);
        startButton.setFocusPainted(false);
        startButton.addFocusListener(this);
        startButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(startButton);           
        
        /* pause button */
        pauseButton = new JToggleButton();
        pauseButton.setIcon(new ImageIcon("images/pause.png"));
        pauseButton.setText("pause");        
        pauseButton.setBorderPainted(false);
        pauseButton.setFocusable(true);
        pauseButton.setFocusPainted(false);
        pauseButton.addFocusListener(this);
        pauseButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(pauseButton);      

        /* stop button */
        stopButton = new JToggleButton();
        stopButton.setIcon(new ImageIcon("images/stop.png"));
        stopButton.setText("stop");        
        stopButton.setBorderPainted(false);
        stopButton.setFocusable(true);
        stopButton.setFocusPainted(false);
        stopButton.addFocusListener(this);
        stopButton.addKeyListener(this);
        stopButton.requestFocusInWindow();         
        /* adds button to the panel */
        buttonsPanel.add(stopButton);        
           
        /* seek-forward. button */
        seekForwardButton = new JToggleButton();
        seekForwardButton.setIcon(new ImageIcon("images/seek-forward.png"));
        seekForwardButton.setText("seek-forward");        
        seekForwardButton.setBorderPainted(false);
        seekForwardButton.setFocusable(true);
        seekForwardButton.setFocusPainted(false);
        seekForwardButton.addFocusListener(this);
        seekForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(seekForwardButton);  

        /* skip-forward button */
        skipForwardButton = new JToggleButton();
        skipForwardButton.setIcon(new ImageIcon("images/skip-forward.png"));
        skipForwardButton.setText("skip-forward");        
        skipForwardButton.setBorderPainted(false);
        skipForwardButton.setFocusable(true);
        skipForwardButton.setFocusPainted(false);
        skipForwardButton.addFocusListener(this);
        skipForwardButton.addKeyListener(this);
        /* adds button to the panel */
        buttonsPanel.add(skipForwardButton);         
    }        

    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */       
    public void focusGained(FocusEvent e) {
        /* paint border the current button */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(true);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */           
    public void focusLost(FocusEvent e) {
        /* erase the border */
        JToggleButton b = (JToggleButton)e.getSource();
        b.setBorderPainted(false);
    }     
    
    /*
     * Invoked when a key is pressed.
     * @param e The key event
     */       
    public void keyPressed(KeyEvent e) {
        int key = e.getKeyCode();
        /* make sure the key isn't processed for anything else */
        e.consume();           
        if (key == KeyEvent.VK_ESCAPE) {            /* QUIT */   
            quitAction();
        } else if (key == KeyEvent.VK_S) {          /* STOP */   
            stopButton.requestFocus();
            stopButton.doClick();
            stopAction();
            stopButton.doClick();
        } else if (key == KeyEvent.VK_L) {         /* PAUSE - RESUME */       
            pauseButton.requestFocus();
            pauseButton.doClick();
            pauseResumeAction();
        } else if (key == KeyEvent.VK_P) {         /* PLAY */       
            startButton.requestFocus();
            startButton.doClick();
            playAction();
            startButton.doClick();
        } else if (key == KeyEvent.VK_C) {         /* SEEK + */     
            seekForwardButton.requestFocus();
            seekForwardButton.doClick();
            seekForwardAction();
            seekForwardButton.doClick();
        } else if (key == KeyEvent.VK_X) {         /* SEEK - */   
            seekBackwardButton.requestFocus();
            seekBackwardButton.doClick();
            seekBackwardAction();
            seekBackwardButton.doClick();
        } else if (key == KeyEvent.VK_U) {         /* VOL UP */    
            volumeUpAction();
        } else if (key == KeyEvent.VK_I) {         /* VOL DOWN */       
           volumeDownAction();          
        } else if (key == KeyEvent.VK_M) {         /* MUTE */       
            muteAction();
        } else if (key == KeyEvent.VK_V) {         /* NEXT */     
            skipForwardButton.requestFocus();
            skipForwardButton.doClick();
            skipForwardAction();
            skipForwardButton.doClick();
        }  else if (key == KeyEvent.VK_W) {         /* PREVIOUS */     
            skipBackwardButton.requestFocus();
            skipBackwardButton.doClick();
            skipBackwardAction();
            skipBackwardButton.doClick();            
        } else if (key == KeyEvent.VK_ENTER) {      /* ACTION KEY */
            JToggleButton button = (JToggleButton) e.getSource();
            String text =button.getText();   
            System.out.println("ACTION BUTON TEXT: " + text);
            if (text == "skip-backward") {
                button.doClick();
                skipBackwardAction();
                button.doClick();
            } else if (text == "seek-backward") {
                button.doClick();
                seekBackwardAction();
                button.doClick();
            } else if (text == "start") {
                button.doClick();
                playAction();
                button.doClick();
            } else if (text == "pause") {
                button.doClick();
                pauseResumeAction();                
            } else if (text == "stop") {
                button.doClick();
                stopAction();                
                button.doClick();
            } else if (text == "seek-forward") {
                button.doClick();
                seekForwardAction();
                button.doClick();
            } else if (text == "skip-forward") {
                button.doClick();
                skipForwardAction();
                button.doClick();
            }
        }
    }

    /*
     * Invoked when a key is released.
     * @param e The key event
     */       
    public void keyReleased(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }
    
    /*
     * Invoked when a key is typed.
     * @param e The key event
     */       
    public void keyTyped(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();                                
    }         
    
    
}

