/*
 * ExternalPlugin.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

/**
 * ExternalPlugin executes a command to run a non-java external plugin.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jul 2007
 */
public class ExternalPlugin implements Runnable {
    
    private Process playerPrc;      
    static BufferedReader stdout, stderr;
    static Vector<String> outputLines;
    private boolean outputAllowed = false;
    private String[] command;
    
    private Thread runner;
    
    /**
     * Constructs the image viewer plugin.
     * @param command  the command to execute.
     */
    ExternalPlugin(String[] command) {
        this.command = command;        
    }            
    
    /**
     * Executes the command.
     * @return Process  the process in execution.
     */    
    public Process executeCommand() {
        try {
	    playerPrc = Runtime.getRuntime().exec(command);          
        } catch (IOException e) {
            e.printStackTrace();
            System.out.println("Dean! Exec failed. Exiting");            
        }
        return playerPrc;
    }
    
    /**
     * Sets the command string.
     * @param command  the command string.
     */      
    public void setCommand(String[] command) {
        this.command = command;
    }
    
    /**
     * Executes this thread.
     */     
    public void run() {
            InputStream is = playerPrc.getInputStream();
            stdout = new BufferedReader(new InputStreamReader(is));                    
            outputLines = new Vector<String>();          
            Thread stdoutThread = new Thread() {
                public void run() {
                    try {
                        int l;
                        String line;
                        //for (l = 0; (line = stdout.readLine()) != null;) {
                        for (l = 0; ((line = stdout.readLine()) != null && runner != null);) {
                            if (line.length() > 0) {
                                l++;
                                outputLines.addElement(line);
                                if (line.matches("Starting playback...")) {
                                    outputAllowed = true;
                                }
                            }
                            System.out.print(outputAllowed ? line +"\n": ".");
                        }
                        System.out.println("\nRead " + l + " lines from stdout.");
                        stdout.close();
                        for (Object o: outputLines) {
                            System.out.println(o);
                        }
                    } catch (IOException ie) {
                        System.out.println("IO exception on stdout: " + ie);
                    }
                }
            };
            
            stdoutThread.start();            
            try {
                playerPrc.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }     
    }
    
    /**
     * Starts the thread calling the run method.
     */ 
    public void start() {
        if (runner == null) {
            runner = new Thread(this);
            runner.start();
        }
    }

    /**
     * Stops the thread avoiding deadlock-prone.
     */     
    public void stop() {
        if (runner != null) {
            Thread temp = runner;
            runner = null;
            temp.interrupt();
        }
    }    
    
}