/*
 * ImagePlugin.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.io.File;
import java.io.IOException;
import javax.imageio.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.BorderLayout;
import java.awt.*;
import java.net.*;

import lhmc.plugins.*;
import lhmc.gui.util.*;


/**
 * ImagePlugin executes a single image file and shows it in fullscreen
 * and executes a directory and make a slideshow from all the images in that
 * directory.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Mar 2007
 */
public class ImagePlugin extends Plugin implements KeyListener {

    /* fullscreen */
    protected GraphicsDevice device;
    protected Window mainWindow;
    
    protected SlideShow ss = null;    
    
    /* needed to avoid serialization warnings */
    private static final long serialVersionUID = 1L;
    
    private JPanel showImagePanel;
    
    /**
     * Constructs the ImagePlugin.
     * @param file  the file to show.
     */
    ImagePlugin() {          
        showImagePanel = new JPanel();
    }    

    /**
     * Shows the scaled image in fullscreen.
     * @param file  the image URL to show.
     */    
    public void play(URL file) {
        /* loads the scaled image */
        
	ScaledImageJComponent showImage = new ScaledImageJComponent(file);
        showImage.setFocusable(true);     
        showImage.addKeyListener(this);
        showImagePanel.add(showImage);
        mainFrame.addToMainScreenCardLayout(showImagePanel, "IMAGE");        
        mainFrame.setMainScreenPanel("IMAGE");
        showImage.requestFocus();                          
    }

    /**
     * Shows a slideshow  of a directory in fullscreen.
     * @param file  the URL directory to make the slideshow.
     */       
    public void playAll(URL file) {
        ss = new SlideShow(file);        

        ss.setFocusable(true);     
        ss.addKeyListener(this);
        //showImagePanel.add(ss);                          
        mainFrame.addToMainScreenCardLayout(ss, "SS");        
        mainFrame.setMainScreenPanel("SS");
        ss.requestFocus();               
        /* starts the slideshow thread */
        ss.start();
    }
    
    
    /**
     * Handles the keyPressed event by invoking the keyPressed methods on listener-a and listener-b.
     * @param e the key event.
     */
    public void keyPressed(KeyEvent e) {     
        int key = e.getKeyCode();        
        if (key == KeyEvent.VK_ESCAPE) {
            if (ss != null ) {
                ss.stop();
            }
            mainFrame.restoreMainScreen(showImagePanel);              
        }
        /* make sure the key isn't processed for anything else */
        e.consume();        
    }
    
    /**
     * Handles the keyReleased event by invoking the keyReleased methods on listener-a and listener-b.
     * @param e the key event.
     */    
    public void keyReleased(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();
    }
    
    /**
     * Handles the keyTyped event by invoking the keyTyped methods on listener-a and listener-b.
     * @param e the key event.
     */    
    public void keyTyped(KeyEvent e) { 
        /* make sure the key isn't processed for anything else */
        e.consume();
    }    
}