/*
 * MencoderPlugin.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.*;          
import java.awt.*;
import java.io.File;
import java.io.IOException;
import javax.imageio.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.BorderLayout;
import java.awt.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import java.text.DateFormat;
import java.util.Date;
import java.util.Calendar;
import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.*;

import lhmc.plugins.*;
import lhmc.utils.*;
import lhmc.gui.util.*;


/**
 * MencoderPlugin records television shows.
 *
 * @author 	Cristina Roura Claver
 * @version 	0.01 Mar 2007
 */
public class MencoderPlugin extends Plugin implements FocusListener, ActionListener {

    /* fullscreen */
    protected GraphicsDevice device;
    protected Window mainWindow;
    
    private Process playerPrc;  
    
    static BufferedReader stdout, stderr;
    static Vector<String> outputLines;
    private boolean paused = false;
    private boolean outputAllowed = false;
        
    /* needed to avoid serialization warnings */
    private static final long serialVersionUID = 1L;

    private JPanel recordJPanel;
    private String channelToRecord = null;
    private String pathToRecord = null;
    
    protected CustomButton focusedButton;

    /* Colors */
    private Color foregroundColor;
    private Color highlightColor;
    private Font labelFont;

    /**
     * Constructs MencoderPlugin.
     * Shows a list of chanels to choose wich to record.
     */
   public MencoderPlugin() {       
	loadColorTheme();
               
        /* creates the panel and the buttons */ 
        JPanel pane = new JPanel();
        pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));
        JLabel title = new JLabel("Choose channel to record: ");
        title.setAlignmentX(Component.CENTER_ALIGNMENT);
        title.setFont(labelFont);
        pane.add(title);
        pane.add(buttonsPanel());
	recordJPanel = pane;        
    }
    
    /**
     * Creates the channels list button panel.
     * @return      the channels panel.
     */
    private JPanel buttonsPanel() {

        
        String[] optionsName = getChannels().split(":");
        
        int numColumns = 3;
        
        /* calculates number of Rows depending on the number of Thumbnails and the default number of columns*/
        int numRows = (int) Math.ceil(1.0 * ( optionsName.length + 1) / numColumns);           
        
        JPanel pane = new JPanel(new GridLayout(numRows, numColumns));
        
         /* buttons creation */
        for (int i = 0 ; i < optionsName.length; i++) {
            /* creates a new button */
            CustomButton button = new CustomButton();
            button.setText(optionsName[i]);
            button.addFocusListener(this);
            button.addActionListener(this);		
            if (i == 0) {       /* is the first button */
                button.requestFocusInWindow(); 
                focusedButton = button;
            }
            pane.add(button);   /* adds button to the panel */
	}
	CustomButton backButton = new CustomButton("Back");
        backButton.addFocusListener(this);
	backButton.addActionListener(this);
        pane.add(backButton);
        return pane;
    }

    /**
     * Shows the panel.
     * @param file  not used.
     */    
    public void play(URL file) {

        //recordJPanel.requestFocusInWindow();
        mainFrame.addToMainScreenCardLayout(recordJPanel, "RECORD");        
        mainFrame.setMainScreenPanel("RECORD");
	focusedButton.requestFocusInWindow();
    }

    /**
     * Records a channel.
     */       
    private void record() {
        //to record analogic tv: mencoder -tv driver=v4l2:width=640:height=480 tv://24 -o tv.avi -ovc lavc -oac pcm        
	//to record dvb:         mplayer -dumpfile r1.ts -dumpstream dvb://laSexta
	//to record dvb:	 mencoder dvb://laSexta -ovc copy -oac copy -o prova.avi

      	String fileName = channelToRecord + "_" + getDateTime() + ".avi";
        /*
        //ANALOGIC
	String[] command = new String[] {
	    "/usr/bin/mencoder",            
	    "-tv", "driver=v4l2:width=640:height=480",
            "tv://1",
            "-ovc", "lavc",
            "-oac", "pcm",
            "-o", pathToRecord + "/" + fileName
	};     
        */
        
        // TDT
	String[] command = new String[] {
	    "/usr/bin/mencoder",            
	    "dvb://" + channelToRecord,
            "-ovc", "copy",
            "-oac", "copy",
            "-o", pathToRecord + "/" + fileName
	};

	System.out.println("COMAND MENCODER: " + channelToRecord);
        
	ExternalPlugin ep = new ExternalPlugin(command);
        ep.executeCommand();
        
        mainFrame.setIsRecordingDvbt(true, ep);
        
        ep.start();                 
    }
     
    public void playAll(URL file) {
    }
    
    
    /**
     * Sets in a string the current time.
     * @return  The time string.
     */    
    private String getDateTime() {              
	Date now = new Date();
	DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.MEDIUM);
	String date = dateFormat.format(now);

        Calendar rightNow = Calendar.getInstance();
        String time = Integer.toString(rightNow.get(Calendar.HOUR_OF_DAY)) + "-" +
                      Integer.toString(rightNow.get(Calendar.MINUTE)) + "-" +
                      Integer.toString(rightNow.get(Calendar.SECOND));
        
        String dateTime = date + "_" + time;
        return dateTime;
    }    

    /**
     * Get the list of channels from the config file Mplayer channels.conf
     * @return  The channels string.
     */     
    private String getChannels() {
        String channels = "";   
        try{     
            // Open the file that is the first 
            // command line parameter
            FileInputStream fstream = new FileInputStream("config/channels.conf");
            // Get the object of DataInputStream
            DataInputStream in = new DataInputStream(fstream);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String strLine;
            //Read File Line By Line
            while ((strLine = br.readLine()) != null)   {
                // Print the content on the console
                //System.out.println (strLine);
                //char[] channel = null;
                //int j=0;
                String channel;
                if (!strLine.startsWith("#")) {
                    String[] line = strLine.split(":");
                    channel = line[0];
                    channels = channels + channel + ":";
                }
            }
            //Close the input stream
            in.close();
        }catch (Exception e){//Catch exception if any
            System.err.println("Error: " + e.getMessage());
        }
        return channels;
}
    
    /**
     * Invoked when the action button pressed occurs.
     * @param e The action event
     */       
    public void actionPerformed(ActionEvent e) { 
	String action = e.getActionCommand();   
        String text = ((CustomButton)e.getSource()).getText();   
        if (text == "Back") {
	    mainFrame.restoreMainScreen(recordJPanel);   
	} else {
            channelToRecord = text;
	    record();
            NoticeDialog notice = new NoticeDialog(mainFrame, "Recording channel: " + text);
            notice.pack();
            notice.setLocationRelativeTo(null);
            notice.setVisible(true);               
            mainFrame.restoreMainScreen(recordJPanel);  
	}
    }
           
    /**
     * Loads the colors from the config file.
     */    
    private void loadColorTheme() {
        ConfigReader config = new ConfigReader();        
        highlightColor = config.loadColor("highlight");        
        foregroundColor = config.loadColor("foreground");       
        pathToRecord = config.getProperty("URLVideoPlayer") + "/recorded tv";
        labelFont = config.loadFont("title");
    }
           
    /*
     * Invoked when a button gains focus.
     * @param e The focus event
     */           
     public void focusGained(FocusEvent e) {
        e.getComponent().setForeground(highlightColor);
    }

    /*
     * Invoked when a button loses focus.
     * @param e The focus event
     */                
    public void focusLost(FocusEvent e) {
        e.getComponent().setForeground(foregroundColor);
    }

}
