/*
 * SlideShow.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.plugins;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.JPanel;
import java.io.*;
import java.net.*;

import jcifs.smb.*;

import lhmc.files.*;

/**
 * Slideshow is a threaded class that shows images in fullscreen form a directory.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jun 2007
 */
public class SlideShow extends JPanel implements Runnable {
    
    /* time between images */
    static final int MILISECONDS = 2000;
    private FileManager fileManager;
    private ScaledImageJComponent si;
    private Thread runner;

    private ImageFileFilter customFileFilter;
    
    static int LOCAL_FILE = 0;
    static int REMOTE_SMBFILE = 1;
    
    private int actualFilesystemLocation = -1;
    
    UrlUtils urlUtils = null;
    
    /**
     * Constructs SlideShow managing the files from the directory.
     * @param file  the url file directory to do the slideshow
     */    
    public SlideShow(URL file) {  
        urlUtils = new UrlUtils(file);
        actualFilesystemLocation = urlUtils.getLocation();
        customFileFilter = new ImageFileFilter();
        URL[] files = urlUtils.getFilteredUrls(file, customFileFilter);
        fileManager = new FileManager(files);          
/*
        jcifs.Config.registerSmbURLHandler();
        String protocol = file.getProtocol();        
        if (protocol.equals("file")) {       
            actualFilesystemLocation = LOCAL_FILE;
        } else if (protocol.equals("smb")) {
            actualFilesystemLocation = REMOTE_SMBFILE;
        }  
        urlUtils = new UrlUtils(actualFilesystemLocation);        
 */
      
      
    }
      
    /**
     * Executes the slideshow, showing an image each MILISECONDS.
     */       
     public void run() {
 
        int numImages = fileManager.getNodesNumber();
        
        for (int i = 0; i < numImages && runner != null; i++) {
            if (!fileManager.isDirectoryNode(i)) {                               
                URL image = fileManager.getFileAt(i);
                si = new ScaledImageJComponent(image);  
                System.out.println("SS: " + image);
                repaint();
                try {
                    Thread.sleep(MILISECONDS);
                } catch (InterruptedException e) {}                                       
            }            
        }
        /* We have reached the end or been stopped */
        runner = null;        
    }
         
     
    /**
     * Starts the thread calling the run method.
     */ 
    public void start() {
        if (runner == null) {
            runner = new Thread(this);
            runner.start();
        }
    }

    /**
     * Stops the thread avoiding deadlock-prone.
     */     
    public void stop() {
        if (runner != null) {
            Thread temp = runner;
            runner = null;
            temp.interrupt();
        }
    }
     
    /* XXX to repair */
    /**
     * Invoques the paintComponent method.
     * @param g the Graphics context.
     */     
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        si.paintScaledImage(g);
    }
}
