/*
 * ConfigReader.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.utils;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.io.*;

/**
 * ConfigReader reads the config file and gets it's properties.
 * 
 * @author 	Cristina Roura Claver
 * @version 	0.01 Jul 2007
 */
public class ConfigReader {

    private Properties prop = new Properties();
        
    /**
     * Constructs ConfigReader.
     */
    public ConfigReader() {            
        /* load properties */
        try {   
            FileInputStream fis = new FileInputStream("config/conf.xml");
            prop.loadFromXML(fis);           
        } catch (FileNotFoundException e) {
            System.out.println("Caught FileNotFoundException " + e);
        } catch (SecurityException e) {
            System.out.println("Caught SecurityException " + e);
        } catch (IOException e) {
            System.out.println("Caught IOException " + e);
        }
    }
    
    /**
     * Gets the property string from key string.
     * @param key   the key string.
     * @return      the property related to the key.
     */    
    public String getProperty(String key) {
        return prop.getProperty(key);        
    }
    
    /**
     * Gets an enumeration on key Names
     * @return      the enumeration of keys.
     */        
    public Enumeration propertyNames() {
        return prop.propertyNames();
    }
    
    /**
     * Gets the color from a colortype string.
     * @param colorType   the name of the color type to get.
     * @return      the Color required.
     */        
    public Color loadColor(String colorType) {
        Color customColor;
        int colorRed = 125;
        int colorGreen = 125;
        int colorBlue = 125;
        
        if (colorType == "foreground") {
            colorRed = Integer.parseInt(prop.getProperty("foregroundColorRed"));
            colorGreen = Integer.parseInt(prop.getProperty("foregroundColorGreen"));
            colorBlue = Integer.parseInt(prop.getProperty("foregroundColorBlue"));            
        } else if (colorType == "highlight") {
            colorRed = Integer.parseInt(prop.getProperty("highlightColorRed"));
            colorGreen = Integer.parseInt(prop.getProperty("highlightColorGreen"));
            colorBlue = Integer.parseInt(prop.getProperty("highlightColorBlue"));            
        } else if (colorType == "selected") {
            colorRed = Integer.parseInt(prop.getProperty("selectedColorRed"));
            colorGreen = Integer.parseInt(prop.getProperty("selectedColorGreen"));
            colorBlue = Integer.parseInt(prop.getProperty("selectedColorBlue"));            
        }
        
        return customColor = new Color(colorRed,colorGreen,colorBlue);
        
    }
    
    /**
     * Gets the font from a fontType string.
     * @param fontType   the name of the font type to get.
     * @return      the Font required.
     */            
    public Font loadFont(String fontType) {
        Font customFont;
        String fontName = prop.getProperty("fontType");
        int fontStyle = Integer.parseInt(prop.getProperty("fontStyle"));
        int fontSize = 20;
        
        if (fontType == "title") {
            fontSize = Integer.parseInt(prop.getProperty("titleFontSize"));                 
        } else if (fontType == "header") {
            fontSize = Integer.parseInt(prop.getProperty("headerFontSize"));
        }  else if (fontType == "button") {
            fontSize = Integer.parseInt(prop.getProperty("buttonFontSize"));
        } else if (fontType == "dialog") {
            fontSize = Integer.parseInt(prop.getProperty("dialogFontSize"));            
        }
        
        return customFont = new Font(fontName, fontStyle, fontSize);
        
    }
}