/*
 * OpticalDiscTypeFinder.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.utils;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * OpticalDiscTypeFinder uses cdtool and dvd+rwtools to discover the disc type.
 *
 * @version 	0.01 Jul 2007
 * @author 	Cristina Roura Claver
 */
public class OpticalDiscTypeFinder {
    
    private int NO_DISC     = 1;
    private int DATA_CD     = 2;    
    private int DATA_DVD    = 3;
    private int AUDIO_CD    = 4;    
    private int VIDEO_DVD   = 5;
    
    private int discType = NO_DISC;
    
    /**
     * Constructs OpticalDiscTypeFinder
     */
    public OpticalDiscTypeFinder() {
        searchingOpticalDisc();
    }
    
    /**
     * Sets the disc type
     * @param type  the disc type.
     */    
    public void setDiscType(int type) {
        discType = type;
    }    
    
    /**
     * Gets the disc type
     * @return  the disc type.
     */       
    public int getDiscType() {
        return discType;
    }
    
    /**
     * Gets the type identification from a string.
     * @param type  the disc string type.
     * @return the disc type.
     */       
   public int getTypeOf(String type) {
       if (type == "NO_DISC") {
           return NO_DISC;
       } else if (type == "DATA_CD") {
           return DATA_CD;           
       } else if (type == "DATA_DVD") {
           return DATA_DVD;           
       } else if (type == "AUDIO_CD") {
           return AUDIO_CD;           
       } else if (type == "VIDEO_DVD") {
           return VIDEO_DVD;
       } else {
           return 0;
       }
   }
    
    /**
     * Searches the optical disc type.
     */       
    private void searchingOpticalDisc() {
        
        /* find out if there is a CD*/
        try {
            String[] command = new String[] { "cdinfo" };
            Process process = Runtime.getRuntime().exec(command);
            InputStream is = process.getInputStream();                                
            BufferedReader stdout = new BufferedReader(new InputStreamReader(is));
            String info = stdout.readLine();
            System.out.println(info);
            if (info.equals("no-status ")) {                    
                setDiscType(AUDIO_CD);
                System.out.println("FOUND AUDIO CD");                                
            } else if (info.equals("data_disc")) {
                System.out.println("FOUND DATA CD");
                setDiscType(DATA_CD);
            }
        } catch (IOException ie) {
            System.out.println("IO exception on stdout: " + ie);
        }
            
        /* fins out if there is a DVD */
        if (discType == NO_DISC) {
            try {
                String[] command = new String[] { "dvd+rw-mediainfo", "/dev/cdrom" };
                Process process = Runtime.getRuntime().exec(command);
                InputStream is = process.getInputStream();

                BufferedReader stdout = new BufferedReader(new InputStreamReader(is));
                String info;
    
                while(( info = stdout.readLine()) != null) {     
                    System.out.println(info);
                    if (info.equals(" Mounted Media:         1Bh, DVD+R")) {
                        System.out.println("SUPOSING DATA DVD");
                        setDiscType(DATA_DVD);
                    } else if (info.equals(" Mounted Media:         10h, DVD-ROM") ||
			       info.equals(" Media ID:              OPTODISC/OP1") ) {
                        System.out.println("FOUND VIDEO DVD");
                        setDiscType(VIDEO_DVD);
                        break;                                                             
                    }
                }    
            } catch (IOException ie) {
                System.out.println("IO exception on stdout: " + ie);
            } 
        }
    }       
}