/*
 * RemovableDevicesActivator.java	
 *
 * Copyright 2007 Cristina Roura Claver cristina.roura@gmail.com
 *
 * This file is part of LHMC.
 *
 * LHMC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * LHMC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package lhmc.utils;

import javax.swing.*;          
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.*;

import lhmc.files.*;

/**
 * RemovableDevicesActivator mounts the removable devices connected to the system.
 * The mount is done automaticaly accecing to the devices by automount.
 *
 * @version 	0.01 Jul 2007
 * @author 	Cristina Roura Claver
 */
public class RemovableDevicesActivator {
    
    private URL devicesRoot = null;
    private URL[] removableDevices = null;
    private UrlUtils urlUtils = null;
    
    /**
     * Constructs RemovableDevicesActivator
     * @param root  the root directory where the devices are mounted.
     */
    public RemovableDevicesActivator(URL root) {
        urlUtils = new UrlUtils(root);
        devicesRoot = root;
        activatingUSBDevices();
    }

    /**
     * Sets the devices root directory.
     * @param root  the root directory where the devices are mounted.
     */    
    public void setDevicesRoot(URL root) {
        devicesRoot = root;
    }
        
    /**
     * Gets the devices root directory.
     * @return  the root directory where the devices are mounted.
     */        
    public URL getDevicesRoot() {
        return devicesRoot;
    }
    
    /**
     * Sets the devices directories URL array.
     * @param devicesList  the devices directory URL array where each device is mounted.
     */     
    public void setRemovableDevices(URL[] devicesList) {
        removableDevices = devicesList;
    }
     
    /**
     * Gets the devices directories URL array.
     * @return  the devices directory URL array where each device is mounted.
     */         
    public URL[] getRemovableDevices() {
        return removableDevices;
    }
    
    /**
     * Activates the removable devices accecing to the files.
     */        
    public void activatingUSBDevices() {
        
        /* activate usb devices */
        for (int i = 1; i <= 7; i++) {
            try {
                String[] command = new String[] { "ls", urlUtils.getPath(devicesRoot) +"/USB" + i };
                Process process = Runtime.getRuntime().exec(command);
            } catch (IOException ie) {
                System.out.println("IO exception on stdout: " + ie);
            }
        }
        try {
            String[] command = new String[] { "ls", urlUtils.getPath(devicesRoot) +"/MS"};
            Process process = Runtime.getRuntime().exec(command);
            command = new String[] { "ls", urlUtils.getPath(devicesRoot) +"/SD_MMC"};
            process = Runtime.getRuntime().exec(command);
            command = new String[] { "ls", urlUtils.getPath(devicesRoot) +"/CF_MD"};
            process = Runtime.getRuntime().exec(command);
            command = new String[] { "ls", urlUtils.getPath(devicesRoot) +"/XD_SM"};
            process = Runtime.getRuntime().exec(command);
        } catch (IOException ie) {
            System.out.println("IO exception on stdout: " + ie);
        }
        try {
            //Pause for 100 microseconds
            Thread.sleep(100);
        } catch (InterruptedException e) {
            System.out.println("InterruptedException: " + e);
        } 
        removableDevices = urlUtils.getFilteredUrls(devicesRoot, null);
        if (removableDevices.length == 0) {
            removableDevices = null;
        }
    }    
}